@@ -1,28 +1,16 @@
 #ifndef MYSYS_MY_HANDLER_ERRORS_INCLUDED
 #define MYSYS_MY_HANDLER_ERRORS_INCLUDED
 
-/* Copyright (c) 2008, 2020, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2008, 2015, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License, version 2.0,
-   as published by the Free Software Foundation.
-
-   This program is also distributed with certain software (including
-   but not limited to OpenSSL) that is licensed under separate terms,
-   as designated in a particular file or component or in included license
-   documentation.  The authors of MySQL hereby grant you an additional
-   permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
-
-   Without limiting anything contained in the foregoing, this file,
-   which is part of C Driver for MySQL (Connector/C), is also subject to the
-   Universal FOSS Exception, version 1.0, a copy of which can be found at
-   http://oss.oracle.com/licenses/universal-foss-exception.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License, version 2.0, for more details.
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software Foundation,
@@ -107,8 +95,17 @@ static const char *handler_error_messages[]=
   "Temporary file write failure",
   "Operation not allowed when innodb_forced_recovery > 0",
   "Too many words in a FTS phrase or proximity search",
-  "Destination schema does not exist",
-  "Too many nested sub-expressions in a full-text search"
+  "Foreign key cascade delete/update exceeds max depth",
+  "Required Create option missing",
+  "Out of memory in storage engine",
+  "Table corrupted",
+  "Query interrupted",
+  "Tablespace cannot be accessed",
+  "Tablespace is not empty",
+  "Incorrect file name",
+  "Operation is not allowed",
+  "Compute generate value failed",
+  "Destination schema does not exist"
 };
 
 extern void my_handler_error_register(void);
