@@ -32,11 +32,10 @@ Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved.
     along with Percona TokuBackup.  If not, see <http://www.gnu.org/licenses/>.
 ======= */
 
-#ident "$Id: 772c98509a3ea7fab5d9ec348ab362a5a0b61090 $"
+#ident "$Id: 1114ba3ad232b3fa2d372033eafdbc661d501840 $"
 
 #include <stdio.h> // rename(),
 #include <fcntl.h> // open()
-#include <unistd.h> // close(), write(), read(), unlink(), truncate(), etc.
 #include <sys/stat.h> // mkdir()
 #include <errno.h>
 #include <dlfcn.h>
@@ -52,7 +51,7 @@ Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved.
 #include "backup_debug.h"
 #include "directory_set.h"
 
-#if defined(DEBUG_HOTBACKUP) && DEBUG_HOTBACKUP
+#if DEBUG_HOTBACKUP
 #define WARN(string, arg) HotBackup::InterposeWarn(string, arg);
 #define TRACE(string, arg) HotBackup::InterposeTrace(string, arg);
 #define ERROR(string, arg) HotBackup::InterposeError(string, arg);
