@@ -251,7 +251,7 @@ static int tokudb_backup_progress_fun(float progress, const char *progress_strin
                                      len,
                                      MYF(MY_FAE+MY_ALLOW_ZERO_PTR)));
     float percentage = progress * 100;
-    int r =
+    int r MY_ATTRIBUTE((unused)) =
         snprintf(be->_the_string,
         len,
         "tokudb backup about %.0f%% done: %s",
@@ -283,8 +283,7 @@ static void tokudb_backup_set_error_string(THD *thd, int error, const char *erro
     char* error_string =
         static_cast<char*>(my_malloc(tokudb_backup_mem_key, n+1, MYF(MY_FAE)));
 
-    int r = snprintf(error_string, n+1, error_fmt, s1, s2, s3);
-
+    int r MY_ATTRIBUTE((unused)) = snprintf(error_string, n+1, error_fmt, s1, s2, s3);
     assert(0 < r && (size_t)r <= n);
     tokudb_backup_set_error(thd, error, error_string);
     my_free(error_string);
@@ -319,7 +318,8 @@ static bool tokudb_backup_check_slave_sql_thread_running(THD *thd) {
          ++it)
     {
         Master_info *mi= it->second;
-        if (mi != NULL && mi->inited && mi->host && mi->host[0]) {
+        compile_time_assert(sizeof(mi->host) / sizeof(void*) > 1);
+        if (mi != NULL && mi->inited && mi->host[0]) {
             have_slave = true;
             scoped_lock_wrapper<BasicLockableMysqlMutextT>
                 with_mi_data_locked_1(BasicLockableMysqlMutextT(
@@ -360,7 +360,8 @@ static bool tokudb_backup_stop_slave_sql_thread(THD *thd) {
              ++it)
         {
             Master_info *mi = it->second;
-            if (mi && mi->inited && mi->host && mi->host[0]) {
+            compile_time_assert(sizeof(mi->host) / sizeof(void*) > 1);
+            if (mi && mi->inited && mi->host[0]) {
                 bool temp_tables_warning = false;
                 have_slave = true;
                 result = !stop_slave(thd, mi, 0, 0, &temp_tables_warning);
@@ -399,7 +400,8 @@ static bool tokudb_backup_start_slave_sql_thread(THD *thd) {
              ++it)
         {
             Master_info *mi= it->second;
-            if (mi && mi->inited && mi->host && mi->host[0]) {
+            compile_time_assert(sizeof(mi->host) / sizeof(void*) > 1);
+            if (mi && mi->inited && mi->host[0]) {
                 have_slave = true;
                 result = !start_slave(thd,
                                       &thd->lex->slave_connection,
@@ -458,11 +460,10 @@ static bool tokudb_backup_wait_for_safe_slave(THD *thd, uint timeout) {
     }
 
     if (!n_attemts &&
-        slave_open_temp_tables.atomic_get()) {
-
-        (sql_thread_started &&
+        slave_open_temp_tables.atomic_get() &&
+        sql_thread_started &&
         !tokudb_backup_check_slave_sql_thread_running(thd) &&
-        !tokudb_backup_start_slave_sql_thread(thd));
+        !tokudb_backup_start_slave_sql_thread(thd)) {
 
         return false;
     }
@@ -577,7 +578,8 @@ static void tokudb_backup_get_master_infos(
          ++it)
     {
         mi= it->second;
-        if (mi != NULL && mi->host && mi->host[0])
+        compile_time_assert(sizeof(mi->host) / sizeof(void*) > 1);
+        if (mi != NULL && mi->host[0])
             tokudb_backup_get_master_info(mi,
                                           executed_gtid_set,
                                           master_info_channels);
