@@ -1,5 +1,7 @@
 /*
-   Copyright (c) 2000, 2015, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2000, 2017, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2018, Percona and/or its affiliates. All rights reserved.
+   Copyright (c) 2009, 2015, MariaDB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -1160,7 +1162,8 @@ write_keys(Sort_param *param, Filesort_info *fs_info, uint count,
       DBUG_RETURN(1);                           /* purecov: inspected */
   }
 
-  if (my_b_write(chunk_file, &merge_chunk, sizeof(merge_chunk)))
+  if (my_b_write(chunk_file, reinterpret_cast<uchar*>(&merge_chunk),
+                 sizeof(merge_chunk)))
     DBUG_RETURN(1);                             /* purecov: inspected */
 
   DBUG_RETURN(0);
@@ -1227,7 +1230,7 @@ void copy_native_longlong(uchar *to, size_t to_length,
 
   @param[in,out] hash    The hash key of the JSON values in the current row.
 */
-static void __attribute__((noinline))
+static void MY_ATTRIBUTE((noinline))
 make_json_sort_key(Item *item, uchar *to, size_t length, ulonglong *hash)
 {
   DBUG_ASSERT(!item->maybe_null || to[-1] == 1);
@@ -1360,7 +1363,7 @@ uint Sort_param::make_sortkey(uchar *to, const uchar *ref_pos)
         if (sort_field->need_strxnfrm)
         {
           char *from=(char*) res->ptr();
-          size_t tmp_length __attribute__((unused));
+          size_t tmp_length MY_ATTRIBUTE((unused));
           if ((uchar*) from == to)
           {
             DBUG_ASSERT(sort_field->length >= length);
@@ -1941,10 +1944,8 @@ uint read_to_buffer(IO_CACHE *fromfile,
                         merge_chunk,
                         static_cast<ulonglong>(merge_chunk->file_position()),
                         static_cast<ulonglong>(bytes_to_read)));
-    if (mysql_file_pread(fromfile->file,
-                         merge_chunk->buffer_start(),
-                         bytes_to_read,
-                         merge_chunk->file_position(), MYF_RW))
+    if (my_b_pread(fromfile, merge_chunk->buffer_start(), bytes_to_read,
+                   merge_chunk->file_position()))
       DBUG_RETURN((uint) -1);			/* purecov: inspected */
 
     size_t num_bytes_read;
@@ -2140,9 +2141,6 @@ int merge_buffers(Sort_param *param, IO_CACHE *from_file,
        Called by Unique::get()
        Copy the first argument to param->unique_buff for unique removal.
        Store it also in 'to_file'.
-
-       This is safe as we know that there is always more than one element
-       in each block to merge (This is guaranteed by the Unique:: algorithm
     */
     merge_chunk= queue.top();
     memcpy(param->unique_buff, merge_chunk->current_key(), rec_length);
@@ -2157,6 +2155,17 @@ int merge_buffers(Sort_param *param, IO_CACHE *from_file,
       error= 0;                                       /* purecov: inspected */
       goto end;                                       /* purecov: inspected */
     }
+    // The top chunk may actually contain only a single element
+    if (merge_chunk->mem_count() == 0)
+    {
+      if (!(error= (int) read_to_buffer(from_file, merge_chunk, param)))
+      {
+        queue.pop();
+        reuse_freed_buff(merge_chunk, &queue);
+      }
+      else if (error == -1)
+        DBUG_RETURN(error);
+    }
     queue.update_top();                   // Top element has been used
   }
 
