@@ -332,6 +332,7 @@ extern ulong connection_errors_peer_addr;
 #endif
 extern ulong log_warnings;
 extern my_bool encrypt_binlog;
+extern my_bool encrypt_tmp_files;
 extern bool  opt_log_syslog_enable;
 extern char *opt_log_syslog_tag;
 #ifndef _WIN32
@@ -342,6 +343,20 @@ extern char *opt_log_syslog_facility;
 extern uint host_cache_size;
 extern ulong log_error_verbosity;
 
+extern bool opt_keyring_operations;
+extern char *opt_keyring_migration_user;
+extern char *opt_keyring_migration_host;
+extern char *opt_keyring_migration_password;
+extern char *opt_keyring_migration_socket;
+extern char *opt_keyring_migration_source;
+extern char *opt_keyring_migration_destination;
+extern ulong opt_keyring_migration_port;
+/**
+  Variable to check if connection related options are set
+  as part of keyring migration.
+*/
+extern bool migrate_connect_options;
+
 /** System variable show_compatibility_56. */
 extern my_bool show_compatibility_56;
 
@@ -444,7 +459,8 @@ extern PSI_mutex_key
   key_structure_guard_mutex, key_TABLE_SHARE_LOCK_ha_data,
   key_LOCK_error_messages,
   key_LOCK_log_throttle_qni, key_LOCK_query_plan, key_LOCK_thd_query,
-  key_LOCK_cost_const, key_LOCK_current_cond;
+  key_LOCK_cost_const, key_LOCK_current_cond,
+  key_LOCK_keyring_operations;
 extern PSI_mutex_key key_RELAYLOG_LOCK_commit;
 extern PSI_mutex_key key_RELAYLOG_LOCK_commit_queue;
 extern PSI_mutex_key key_RELAYLOG_LOCK_done;
@@ -829,6 +845,7 @@ extern size_t mysql_real_data_home_len;
 extern const char *mysql_real_data_home_ptr;
 extern MYSQL_PLUGIN_IMPORT char  *mysql_data_home;
 extern "C" MYSQL_PLUGIN_IMPORT char server_version[SERVER_VERSION_LENGTH];
+extern "C" MYSQL_PLUGIN_IMPORT char server_version_suffix[SERVER_VERSION_LENGTH];
 extern MYSQL_PLUGIN_IMPORT char mysql_real_data_home[];
 extern char mysql_unpacked_real_data_home[];
 extern MYSQL_PLUGIN_IMPORT struct system_variables global_system_variables;
@@ -868,6 +885,7 @@ extern mysql_rwlock_t LOCK_consistent_snapshot;
 extern mysql_cond_t COND_manager;
 extern int32 thread_running;
 extern mysql_mutex_t LOCK_group_replication_handler;
+extern mysql_mutex_t LOCK_keyring_operations;
 
 extern char *opt_ssl_ca, *opt_ssl_capath, *opt_ssl_cert, *opt_ssl_cipher,
             *opt_ssl_key, *opt_ssl_crl, *opt_ssl_crlpath, *opt_tls_version;
@@ -946,7 +964,14 @@ enum options_mysqld
   OPT_SHOW_OLD_TEMPORALS,
   OPT_ENFORCE_GTID_CONSISTENCY,
   OPT_TRANSACTION_READ_ONLY,
-  OPT_TRANSACTION_ISOLATION
+  OPT_TRANSACTION_ISOLATION,
+  OPT_KEYRING_MIGRATION_SOURCE,
+  OPT_KEYRING_MIGRATION_DESTINATION,
+  OPT_KEYRING_MIGRATION_USER,
+  OPT_KEYRING_MIGRATION_HOST,
+  OPT_KEYRING_MIGRATION_PASSWORD,
+  OPT_KEYRING_MIGRATION_SOCKET,
+  OPT_KEYRING_MIGRATION_PORT
 };
 
 
