@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2017, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2015, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -10,12 +10,8 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
-
-#ifdef USE_PRAGMA_INTERFACE
-#pragma implementation /* gcc class implementation */
-#endif
+   along with this program; if not, write to the Free Software Foundation,
+   51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA */
 
 /**
   @file
@@ -25,22 +21,20 @@
 */
 
 #include "records.h"
-#include "sql_priv.h"
-#include "records.h"
+#include "sql_list.h"
 #include "filesort.h"            // filesort_free_buffers
-#include "opt_range.h"                          // SQL_SELECT
 #include "sql_class.h"                          // THD
+#include "sql_select.h"          // JOIN_TAB
 
 
 static int rr_quick(READ_RECORD *info);
 int rr_sequential(READ_RECORD *info);
 static int rr_from_tempfile(READ_RECORD *info);
-static int rr_unpack_from_tempfile(READ_RECORD *info);
-static int rr_unpack_from_buffer(READ_RECORD *info);
+template<bool> static int rr_unpack_from_tempfile(READ_RECORD *info);
+template<bool> static int rr_unpack_from_buffer(READ_RECORD *info);
 static int rr_from_pointers(READ_RECORD *info);
 static int rr_from_cache(READ_RECORD *info);
 static int init_rr_cache(THD *thd, READ_RECORD *info);
-static int rr_cmp(uchar *a,uchar *b);
 static int rr_index_first(READ_RECORD *info);
 static int rr_index_last(READ_RECORD *info);
 static int rr_index(READ_RECORD *info);
@@ -62,17 +56,19 @@ static int rr_index_desc(READ_RECORD *info);
                       occurs (except for end-of-records error)
   @param idx          index to scan
   @param reverse      Scan in the reverse direction
+
+  @retval true   error
+  @retval false  success
 */
 
-void init_read_record_idx(READ_RECORD *info, THD *thd, TABLE *table,
+bool init_read_record_idx(READ_RECORD *info, THD *thd, TABLE *table,
                           bool print_error, uint idx, bool reverse)
 {
   int error;
   empty_record(table);
-  bzero((char*) info,sizeof(*info));
+  memset(static_cast<void*>(info), 0, sizeof(*info));
   info->thd= thd;
   info->table= table;
-  info->file=  table->file;
   info->record= table->record[0];
   info->print_error= print_error;
   info->unlock_row= rr_unlock_row;
@@ -83,10 +79,12 @@ void init_read_record_idx(READ_RECORD *info, THD *thd, TABLE *table,
   {
     if (print_error)
       table->file->print_error(error, MYF(0));
+    return true;
   }
 
   /* read_record will be changed to rr_index in rr_index_first */
   info->read_record= reverse ? rr_index_last : rr_index_first;
+  return false;
 }
 
 
@@ -110,9 +108,10 @@ void init_read_record_idx(READ_RECORD *info, THD *thd, TABLE *table,
     init_read_record()
       info              OUT read structure
       thd               Thread handle
-      table             Table the data [originally] comes from.
-      select            SQL_SELECT structure. We may select->quick or 
-                        select->file as data source
+      table             Table the data [originally] comes from; if NULL,
+      'table' is inferred from 'qep_tab'; if non-NULL, 'qep_tab' must be NULL.
+      qep_tab           QEP_TAB for 'table', if there is one; we may use
+      qep_tab->quick() as data source
       use_record_cache  Call file->extra_opt(HA_EXTRA_CACHE,...)
                         if we're going to do sequential read and some
                         additional conditions are satisfied.
@@ -173,30 +172,38 @@ void init_read_record_idx(READ_RECORD *info, THD *thd, TABLE *table,
   rr_sequential:
   --------------
     This is the most basic access method of a table using rnd_init,
-    rnd_next and rnd_end. No indexes are used.
+    ha_rnd_next and rnd_end. No indexes are used.
+
+  @retval true   error
+  @retval false  success
 */
-void init_read_record(READ_RECORD *info,THD *thd, TABLE *table,
-		      SQL_SELECT *select,
+bool init_read_record(READ_RECORD *info,THD *thd,
+                      TABLE *table, QEP_TAB *qep_tab,
 		      int use_record_cache, bool print_error, 
                       bool disable_rr_cache)
 {
+  int error= 0;
   IO_CACHE *tempfile;
   DBUG_ENTER("init_read_record");
 
-  bzero((char*) info,sizeof(*info));
+  // If only 'table' is given, assume no quick, no condition.
+  DBUG_ASSERT(!(table && qep_tab));
+  if (!table)
+    table= qep_tab->table();
+
+  memset(static_cast<void*>(info), 0, sizeof(*info));
   info->thd=thd;
   info->table=table;
-  info->file= table->file;
   info->forms= &info->table;		/* Only one table */
   
   if (table->s->tmp_table == NON_TRANSACTIONAL_TMP_TABLE &&
-      !table->sort.addon_field)
+      !table->sort.using_addon_fields())
     (void) table->file->extra(HA_EXTRA_MMAP);
   
-  if (table->sort.addon_field)
+  if (table->sort.using_addon_fields())
   {
-    info->rec_buf= table->sort.addon_buf;
-    info->ref_length= table->sort.addon_length;
+    info->rec_buf= table->sort.addon_fields->get_addon_buf();
+    info->ref_length= table->sort.addon_fields->get_addon_buf_length();
   }
   else
   {
@@ -204,15 +211,13 @@ void init_read_record(READ_RECORD *info,THD *thd, TABLE *table,
     info->record= table->record[0];
     info->ref_length= table->file->ref_length;
   }
-  info->select=select;
+  info->quick= qep_tab ? qep_tab->quick() : NULL;
   info->print_error=print_error;
   info->unlock_row= rr_unlock_row;
   info->ignore_not_found_rows= 0;
   table->status=0;			/* And it's always found */
 
-  if (select && my_b_inited(&select->file))
-    tempfile= &select->file;
-  else if (select && select->quick && select->quick->clustered_pk_range())
+  if (info->quick && info->quick->clustered_pk_range())
   {
     /*
       In case of QUICK_INDEX_MERGE_SELECT with clustered pk range we have to
@@ -225,14 +230,27 @@ void init_read_record(READ_RECORD *info,THD *thd, TABLE *table,
     tempfile= table->sort.io_cache;
   if (tempfile && my_b_inited(tempfile)) // Test if ref-records was used
   {
-    DBUG_PRINT("info",("using rr_from_tempfile"));
-    info->read_record= (table->sort.addon_field ?
-                        rr_unpack_from_tempfile : rr_from_tempfile);
+    if (table->sort.using_addon_fields())
+    {
+      DBUG_PRINT("info",("using rr_unpack_from_tempfile"));
+      if (table->sort.addon_fields->using_packed_addons())
+        info->read_record= rr_unpack_from_tempfile<true>;
+      else
+        info->read_record= rr_unpack_from_tempfile<false>;
+    }
+    else
+    {
+      DBUG_PRINT("info",("using rr_from_tempfile"));
+      info->read_record= rr_from_tempfile;
+    }
+
     info->io_cache=tempfile;
-    reinit_io_cache(info->io_cache,READ_CACHE,0L,0,0);
+    if (reinit_io_cache(info->io_cache, READ_CACHE, 0L, 0, 0))
+      goto err;
     info->ref_pos=table->file->ref;
-    if (!table->file->inited)
-      table->file->ha_rnd_init(0);
+    if (!table->file->inited &&
+        (error= table->file->ha_rnd_init(0)))
+      goto err;
 
     /*
       table->sort.addon_field is checked because if we use addon fields,
@@ -240,8 +258,7 @@ void init_read_record(READ_RECORD *info,THD *thd, TABLE *table,
       and table->sort.io_cache is read sequentially
     */
     if (!disable_rr_cache &&
-        !table->sort.addon_field &&
-        ! (specialflag & SPECIAL_SAFE_MODE) &&
+        !table->sort.using_addon_fields() &&
 	thd->variables.read_rnd_buff_size &&
 	!(table->file->ha_table_flags() & HA_FAST_KEY_READ) &&
 	(table->db_stat & HA_READ_ONLY ||
@@ -254,33 +271,49 @@ void init_read_record(READ_RECORD *info,THD *thd, TABLE *table,
 	!table->s->blob_fields &&
         info->ref_length <= MAX_REFLENGTH)
     {
-      if (! init_rr_cache(thd, info))
-      {
-	DBUG_PRINT("info",("using rr_from_cache"));
-	info->read_record=rr_from_cache;
-      }
+      if (init_rr_cache(thd, info))
+        goto skip_caching;
+      DBUG_PRINT("info",("using rr_from_cache"));
+      info->read_record=rr_from_cache;
     }
   }
-  else if (select && select->quick)
+  else if (info->quick)
   {
     DBUG_PRINT("info",("using rr_quick"));
     info->read_record=rr_quick;
   }
-  else if (table->sort.record_pointers)
+  // See save_index() which stores the filesort result set.
+  else if (table->sort.has_filesort_result_in_memory())
   {
-    DBUG_PRINT("info",("using record_pointers"));
-    table->file->ha_rnd_init(0);
-    info->cache_pos=table->sort.record_pointers;
-    info->cache_end=info->cache_pos+ 
-                    table->sort.found_records*info->ref_length;
-    info->read_record= (table->sort.addon_field ?
-                        rr_unpack_from_buffer : rr_from_pointers);
+    if ((error= table->file->ha_rnd_init(0)))
+      goto err;
+
+    info->cache_pos=table->sort.sorted_result;
+    if (table->sort.using_addon_fields())
+    {
+      DBUG_PRINT("info",("using rr_unpack_from_buffer"));
+      DBUG_ASSERT(table->sort.sorted_result_in_fsbuf);
+      info->unpack_counter= 0;
+      if (table->sort.addon_fields->using_packed_addons())
+        info->read_record= rr_unpack_from_buffer<true>;
+      else
+        info->read_record= rr_unpack_from_buffer<false>;
+      info->cache_end= table->sort.sorted_result_end;
+    }
+    else
+    {
+      DBUG_PRINT("info",("using rr_from_pointers"));
+      info->read_record= rr_from_pointers;
+      info->cache_end=
+        info->cache_pos + table->sort.found_records * info->ref_length;
+    }
   }
   else
   {
     DBUG_PRINT("info",("using rr_sequential"));
     info->read_record=rr_sequential;
-    table->file->ha_rnd_init(1);
+    if ((error= table->file->ha_rnd_init(1)))
+      goto err;
     /* We can use record cache if we don't update dynamic length tables */
     if (!table->no_cache &&
 	(use_record_cache > 0 ||
@@ -291,15 +324,27 @@ void init_read_record(READ_RECORD *info,THD *thd, TABLE *table,
       (void) table->file->extra_opt(HA_EXTRA_CACHE,
 				  thd->variables.read_buff_size);
   }
-  /* Condition pushdown to storage engine */
-  if ((thd->variables.optimizer_switch &
-       OPTIMIZER_SWITCH_ENGINE_CONDITION_PUSHDOWN) && 
-      select && select->cond && 
-      (select->cond->used_tables() & table->map) &&
-      !table->file->pushed_cond)
-    table->file->cond_push(select->cond);
 
-  DBUG_VOID_RETURN;
+skip_caching:
+  /*
+    Do condition pushdown for UPDATE/DELETE.
+    TODO: Remove this from here as it causes two condition pushdown calls
+    when we're running a SELECT and the condition cannot be pushed down.
+    Some temporary tables do not have a TABLE_LIST object, and it is never
+    needed to push down conditions (ECP) for such tables.
+  */
+  if (thd->optimizer_switch_flag(OPTIMIZER_SWITCH_ENGINE_CONDITION_PUSHDOWN) &&
+      qep_tab && qep_tab->condition() && table->pos_in_table_list &&
+      (qep_tab->condition()->used_tables() & table->pos_in_table_list->map()) &&
+      !table->file->pushed_cond)
+    table->file->cond_push(qep_tab->condition());
+
+  DBUG_RETURN(false);
+
+err:
+  if (print_error)
+    table->file->print_error(error, MYF(0));     
+  DBUG_RETURN(true);
 } /* init_read_record */
 
 
@@ -308,15 +353,19 @@ void end_read_record(READ_RECORD *info)
 {                   /* free cache if used */
   if (info->cache)
   {
-    my_free_lock(info->cache);
+    my_free(info->cache);
     info->cache=0;
   }
-  if (info->table)
+  if (info->table && info->table->key_read)
+  {
+    info->table->set_keyread(FALSE);
+  }
+  if (info->table && info->table->is_created())
   {
     filesort_free_buffers(info->table,0);
-    (void) info->file->extra(HA_EXTRA_NO_CACHE);
+    (void) info->table->file->extra(HA_EXTRA_NO_CACHE);
     if (info->read_record != rr_quick) // otherwise quick_range does it
-      (void) info->file->ha_index_or_rnd_end();
+      (void) info->table->file->ha_index_or_rnd_end();
     info->table=0;
   }
 }
@@ -347,7 +396,7 @@ static int rr_handle_error(READ_RECORD *info, int error)
 static int rr_quick(READ_RECORD *info)
 {
   int tmp;
-  while ((tmp= info->select->quick->get_next()))
+  while ((tmp= info->quick->get_next()))
   {
     if (info->thd->killed || (tmp != HA_ERR_RECORD_DELETED))
     {
@@ -355,6 +404,7 @@ static int rr_quick(READ_RECORD *info)
       break;
     }
   }
+
   return tmp;
 }
 
@@ -374,8 +424,13 @@ static int rr_quick(READ_RECORD *info)
 
 static int rr_index_first(READ_RECORD *info)
 {
-  int tmp= info->file->index_first(info->record);
+  int tmp= info->table->file->prepare_index_scan();
   info->read_record= rr_index;
+  if (tmp) {
+    tmp= rr_handle_error(info, tmp);
+    return tmp;
+  }
+  tmp= info->table->file->ha_index_first(info->record);
   if (tmp)
     tmp= rr_handle_error(info, tmp);
   return tmp;
@@ -397,8 +452,13 @@ static int rr_index_first(READ_RECORD *info)
 
 static int rr_index_last(READ_RECORD *info)
 {
-  int tmp= info->file->index_last(info->record);
+  int tmp= info->table->file->prepare_index_scan();
   info->read_record= rr_index_desc;
+  if (tmp) {
+    tmp= rr_handle_error(info, tmp);
+    return tmp;
+  }
+  tmp= info->table->file->ha_index_last(info->record);
   if (tmp)
     tmp= rr_handle_error(info, tmp);
   return tmp;
@@ -423,7 +483,7 @@ static int rr_index_last(READ_RECORD *info)
 
 static int rr_index(READ_RECORD *info)
 {
-  int tmp= info->file->index_next(info->record);
+  int tmp= info->table->file->ha_index_next(info->record);
   if (tmp)
     tmp= rr_handle_error(info, tmp);
   return tmp;
@@ -448,7 +508,7 @@ static int rr_index(READ_RECORD *info)
 
 static int rr_index_desc(READ_RECORD *info)
 {
-  int tmp= info->file->index_prev(info->record);
+  int tmp= info->table->file->ha_index_prev(info->record);
   if (tmp)
     tmp= rr_handle_error(info, tmp);
   return tmp;
@@ -458,10 +518,10 @@ static int rr_index_desc(READ_RECORD *info)
 int rr_sequential(READ_RECORD *info)
 {
   int tmp;
-  while ((tmp=info->file->rnd_next(info->record)))
+  while ((tmp=info->table->file->ha_rnd_next(info->record)))
   {
     /*
-      rnd_next can return RECORD_DELETED for MyISAM when one thread is
+      ha_rnd_next can return RECORD_DELETED for MyISAM when one thread is
       reading and another deleting without locks.
     */
     if (info->thd->killed || (tmp != HA_ERR_RECORD_DELETED))
@@ -481,7 +541,7 @@ static int rr_from_tempfile(READ_RECORD *info)
   {
     if (my_b_read(info->io_cache,info->ref_pos,info->ref_length))
       return -1;					/* End of file */
-    if (!(tmp=info->file->rnd_pos(info->record,info->ref_pos)))
+    if (!(tmp=info->table->file->ha_rnd_pos(info->record,info->ref_pos)))
       break;
     /* The following is extremely unlikely to happen */
     if (tmp == HA_ERR_RECORD_DELETED ||
@@ -494,6 +554,30 @@ static int rr_from_tempfile(READ_RECORD *info)
 } /* rr_from_tempfile */
 
 
+template<bool Packed_addon_fields>
+inline void Filesort_info::unpack_addon_fields(uchar *buff)
+{
+  Sort_addon_field *addonf= addon_fields->begin();
+
+  const uchar *start_of_record= buff + addonf->offset;
+
+  for ( ; addonf != addon_fields->end(); ++addonf)
+  {
+    Field *field= addonf->field;
+    if (addonf->null_bit && (addonf->null_bit & buff[addonf->null_offset]))
+    {
+      field->set_null();
+      continue;
+    }
+    field->set_notnull();
+    if (Packed_addon_fields)
+      start_of_record= field->unpack(field->ptr, start_of_record);
+    else
+      field->unpack(field->ptr, buff + addonf->offset);
+  }
+}
+
+
 /**
   Read a result set record from a temporary file after sorting.
 
@@ -503,19 +587,45 @@ static int rr_from_tempfile(READ_RECORD *info)
   positions in the regular record buffer.
 
   @param info          Reference to the context including record descriptors
-
+  @tparam Packed_addon_fields Are the addon fields packed?
+     This is a compile-time constant, to avoid if (....) tests during execution.
   @retval
     0   Record successfully read.
   @retval
     -1   There is no record to be read anymore.
 */
-
+template<bool Packed_addon_fields>
 static int rr_unpack_from_tempfile(READ_RECORD *info)
 {
-  if (my_b_read(info->io_cache, info->rec_buf, info->ref_length))
-    return -1;
-  TABLE *table= info->table;
-  (*table->sort.unpack)(table->sort.addon_field, info->rec_buf);
+  uchar *destination= info->rec_buf;
+#ifndef DBUG_OFF
+  my_off_t where= my_b_tell(info->io_cache);
+#endif
+  if (Packed_addon_fields)
+  {
+    const uint len_sz= Addon_fields::size_of_length_field;
+
+    // First read length of the record.
+    if (my_b_read(info->io_cache, destination, len_sz))
+      return -1;
+    uint res_length= Addon_fields::read_addon_length(destination);
+    DBUG_PRINT("info", ("rr_unpack from %llu to %p sz %u",
+                        static_cast<ulonglong>(where),
+                        destination, res_length));
+    DBUG_ASSERT(res_length > len_sz);
+    DBUG_ASSERT(info->table->sort.using_addon_fields());
+
+    // Then read the rest of the record.
+    if (my_b_read(info->io_cache, destination + len_sz, res_length - len_sz))
+      return -1;                                /* purecov: inspected */
+  }
+  else
+  {
+    if (my_b_read(info->io_cache, destination, info->ref_length))
+      return -1;
+  }
+
+  info->table->sort.unpack_addon_fields<Packed_addon_fields>(destination);
 
   return 0;
 }
@@ -532,7 +642,7 @@ static int rr_from_pointers(READ_RECORD *info)
     cache_pos= info->cache_pos;
     info->cache_pos+= info->ref_length;
 
-    if (!(tmp=info->file->rnd_pos(info->record,cache_pos)))
+    if (!(tmp=info->table->file->ha_rnd_pos(info->record,cache_pos)))
       break;
 
     /* The following is extremely unlikely to happen */
@@ -548,36 +658,47 @@ static int rr_from_pointers(READ_RECORD *info)
 /**
   Read a result set record from a buffer after sorting.
 
-  The function first reads the next sorted record from the sort buffer.
-  If a success it calls a callback function that unpacks 
-  the fields values use in the result set from this buffer into their
-  positions in the regular record buffer.
+  Get the next record from the filesort buffer,
+  then unpack the fields into their positions in the regular record buffer.
 
   @param info          Reference to the context including record descriptors
+  @tparam Packed_addon_fields Are the addon fields packed?
+     This is a compile-time constant, to avoid if (....) tests during execution.
 
   @retval
     0   Record successfully read.
   @retval
     -1   There is no record to be read anymore.
 */
-
+template<bool Packed_addon_fields>
 static int rr_unpack_from_buffer(READ_RECORD *info)
 {
-  if (info->cache_pos == info->cache_end)
+  if (info->unpack_counter == info->table->sort.found_records)
     return -1;                      /* End of buffer */
-  TABLE *table= info->table;
-  (*table->sort.unpack)(table->sort.addon_field, info->cache_pos);
-  info->cache_pos+= info->ref_length;
 
+  uchar *record= info->table->sort.get_sorted_record(
+    static_cast<uint>(info->unpack_counter));
+  uchar *plen= record + info->table->sort.get_sort_length();
+  info->table->sort.unpack_addon_fields<Packed_addon_fields>(plen);
+  info->unpack_counter++;
   return 0;
 }
 	/* cacheing of records from a database */
 
+/**
+  Initialize caching of records from temporary file.
+  
+  @retval
+    0 OK, use caching.
+    1 Buffer is too small, or cannot be allocated.
+      Skip caching, and read records directly from temporary file.
+ */
 static int init_rr_cache(THD *thd, READ_RECORD *info)
 {
   uint rec_cache_size;
   DBUG_ENTER("init_rr_cache");
 
+  READ_RECORD info_copy= *info;
   info->struct_length= 3+MAX_REFLENGTH;
   info->reclength= ALIGN_SIZE(info->table->s->reclength+1);
   if (info->reclength < info->struct_length)
@@ -590,14 +711,14 @@ static int init_rr_cache(THD *thd, READ_RECORD *info)
   info->rec_cache_size= info->cache_records*info->ref_length;
 
   if (info->cache_records <= 2 ||
-      !(info->cache=(uchar*) my_malloc_lock(rec_cache_size+info->cache_records*
-					   info->struct_length,
-					   MYF(0))))
+      !(info->cache=(uchar*) my_malloc(key_memory_READ_RECORD_cache,
+                                       rec_cache_size+info->cache_records*
+                                       info->struct_length,
+                                       MYF(0))))
+  {
+    *info= info_copy;
     DBUG_RETURN(1);
-#ifdef HAVE_purify
-  // Avoid warnings in qsort
-  bzero(info->cache,rec_cache_size+info->cache_records* info->struct_length);
-#endif
+  }
   DBUG_PRINT("info",("Allocated buffert for %d records",info->cache_records));
   info->read_positions=info->cache+rec_cache_size;
   info->cache_pos=info->cache_end=info->cache;
@@ -605,6 +726,14 @@ static int init_rr_cache(THD *thd, READ_RECORD *info)
 } /* init_rr_cache */
 
 
+static int rr_cmp(const void *p_ref_length, const void *a, const void *b)
+{
+  size_t ref_length= *(static_cast<size_t*>(const_cast<void*>(p_ref_length)));
+  DBUG_ASSERT(ref_length <= MAX_REFLENGTH);
+  return memcmp(a, b, ref_length);
+}
+
+
 static int rr_from_cache(READ_RECORD *info)
 {
   uint i;
@@ -620,7 +749,7 @@ static int rr_from_cache(READ_RECORD *info)
     {
       if (info->cache_pos[info->error_offset])
       {
-	shortget(error,info->cache_pos);
+	shortget(&error, info->cache_pos);
 	if (info->print_error)
 	  info->table->file->print_error(error,MYF(0));
       }
@@ -653,8 +782,9 @@ static int rr_from_cache(READ_RECORD *info)
       int3store(ref_position,(long) i);
       ref_position+=3;
     }
-    my_qsort(info->read_positions, length, info->struct_length,
-             (qsort_cmp) rr_cmp);
+    size_t ref_length= info->ref_length;
+    my_qsort2(info->read_positions, length, info->struct_length,
+              rr_cmp, &ref_length);
 
     position=info->read_positions;
     for (i=0 ; i < length ; i++)
@@ -664,12 +794,13 @@ static int rr_from_cache(READ_RECORD *info)
       record=uint3korr(position);
       position+=3;
       record_pos=info->cache+record*info->reclength;
-      if ((error=(int16) info->file->rnd_pos(record_pos,info->ref_pos)))
+      error= (int16) info->table->file->ha_rnd_pos(record_pos, info->ref_pos);
+      if (error)
       {
 	record_pos[info->error_offset]=1;
 	shortstore(record_pos,error);
 	DBUG_PRINT("error",("Got error: %d:%d when reading row",
-			    my_errno, error));
+			    my_errno(), (int) error));
       }
       else
 	record_pos[info->error_offset]=0;
@@ -679,25 +810,15 @@ static int rr_from_cache(READ_RECORD *info)
 } /* rr_from_cache */
 
 
-static int rr_cmp(uchar *a,uchar *b)
+/**
+  The default implementation of unlock-row method of READ_RECORD,
+  used in all access methods.
+*/
+
+void rr_unlock_row(QEP_TAB *tab)
 {
-  if (a[0] != b[0])
-    return (int) a[0] - (int) b[0];
-  if (a[1] != b[1])
-    return (int) a[1] - (int) b[1];
-  if (a[2] != b[2])
-    return (int) a[2] - (int) b[2];
-#if MAX_REFLENGTH == 4
-  return (int) a[3] - (int) b[3];
-#else
-  if (a[3] != b[3])
-    return (int) a[3] - (int) b[3];
-  if (a[4] != b[4])
-    return (int) a[4] - (int) b[4];
-  if (a[5] != b[5])
-    return (int) a[5] - (int) b[5];
-  if (a[6] != b[6])
-    return (int) a[6] - (int) b[6];
-  return (int) a[7] - (int) b[7];
-#endif
+  READ_RECORD *info= &tab->read_record;
+  info->table->file->unlock_row();
 }
+
+
