@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2000, 2013, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2000, 2016, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,8 +11,8 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+   along with this program; if not, write to the Free Software Foundation,
+   51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA */
 
 
 /* Analyse database */
@@ -23,39 +23,34 @@
 **	 - type set is out of optimization yet
 */
 
-#ifdef USE_PRAGMA_IMPLEMENTATION
-#pragma implementation				// gcc: Class implementation
-#endif
-
-#define MYSQL_LEX 1
-
-#include "sql_priv.h"
-#include "procedure.h"
 #include "sql_analyse.h"
-#include <m_ctype.h>
 
-#define MAX_TREEMEM	  8192
-#define MAX_TREE_ELEMENTS 256
+#include "procedure.h"       // Item_proc
+#include "sql_yacc.h"        // DECIMAL_NUM
 
-int sortcmp2(void* cmp_arg __attribute__((unused)),
+#include <algorithm>
+using std::min;
+using std::max;
+
+int sortcmp2(void* cmp_arg MY_ATTRIBUTE((unused)),
 	     const String *a,const String *b)
 {
   return sortcmp(a,b,a->charset());
 }
 
-int compare_double2(void* cmp_arg __attribute__((unused)),
+int compare_double2(void* cmp_arg MY_ATTRIBUTE((unused)),
 		    const double *s, const double *t)
 {
   return compare_double(s,t);
 }
 
-int compare_longlong2(void* cmp_arg __attribute__((unused)),
+int compare_longlong2(void* cmp_arg MY_ATTRIBUTE((unused)),
 		      const longlong *s, const longlong *t)
 {
   return compare_longlong(s,t);
 }
 
-int compare_ulonglong2(void* cmp_arg __attribute__((unused)),
+int compare_ulonglong2(void* cmp_arg MY_ATTRIBUTE((unused)),
 		       const ulonglong *s, const ulonglong *t)
 {
   return compare_ulonglong(s,t);
@@ -66,82 +61,30 @@ int compare_decimal2(int* len, const char *s, const char *t)
   return memcmp(s, t, *len);
 }
 
+/**
+  Create column data accumulator structures 
 
-Procedure *
-proc_analyse_init(THD *thd, ORDER *param, select_result *result,
-		  List<Item> &field_list)
+  @param field_list     Output columns of the original SELECT
+
+  @retval false         Success
+  @retval true          Failure (OOM)
+*/
+bool
+Query_result_analyse::init(List<Item> &field_list)
 {
-  char *proc_name = (*param->item)->name;
-  analyse *pc = new analyse(result);
-  field_info **f_info;
   DBUG_ENTER("proc_analyse_init");
 
-  if (!pc)
-    DBUG_RETURN(0);
+  DBUG_ASSERT(thd->lex->sql_command == SQLCOM_SELECT);
 
-  if (!(param = param->next))
-  {
-    pc->max_tree_elements = MAX_TREE_ELEMENTS;
-    pc->max_treemem = MAX_TREEMEM;
-  }
-  else if (param->next)
-  {
-    // first parameter
-    if (!(*param->item)->fixed && (*param->item)->fix_fields(thd, param->item))
-    {
-      DBUG_PRINT("info", ("fix_fields() for the first parameter failed"));
-      goto err;
-    }
-    if ((*param->item)->type() != Item::INT_ITEM ||
-	(*param->item)->val_real() < 0)
-    {
-      my_error(ER_WRONG_PARAMETERS_TO_PROCEDURE, MYF(0), proc_name);
-      goto err;
-    }
-    pc->max_tree_elements = (uint) (*param->item)->val_int();
-    param = param->next;
-    if (param->next)  // no third parameter possible
-    {
-      my_error(ER_WRONG_PARAMCOUNT_TO_PROCEDURE, MYF(0), proc_name);
-      goto err;
-    }
-    // second parameter
-    if (!(*param->item)->fixed && (*param->item)->fix_fields(thd, param->item))
-    {
-      DBUG_PRINT("info", ("fix_fields() for the second parameter failed"));
-      goto err;
-    }
-    if ((*param->item)->type() != Item::INT_ITEM ||
-	(*param->item)->val_real() < 0)
-    {
-      my_error(ER_WRONG_PARAMETERS_TO_PROCEDURE, MYF(0), proc_name);
-      goto err;
-    }
-    pc->max_treemem = (uint) (*param->item)->val_int();
-  }
-  else if ((*param->item)->type() != Item::INT_ITEM ||
-	   (*param->item)->val_real() < 0)
-  {
-    my_error(ER_WRONG_PARAMETERS_TO_PROCEDURE, MYF(0), proc_name);
-    goto err;
-  }
-  // if only one parameter was given, it will be the value of max_tree_elements
-  else
-  {
-    pc->max_tree_elements = (uint) (*param->item)->val_int();
-    pc->max_treemem = MAX_TREEMEM;
-  }
-
-  if (!(pc->f_info=
+  if (!(f_info=
         (field_info**)sql_alloc(sizeof(field_info*)*field_list.elements)))
-    goto err;
-  pc->f_end = pc->f_info + field_list.elements;
-  pc->fields = field_list;
+    DBUG_RETURN(true);
+
+  f_end= f_info + field_list.elements;
 
   {
-    List_iterator_fast<Item> it(pc->fields);
-    f_info = pc->f_info;
-
+    List_iterator_fast<Item> it(field_list);
+    field_info **info= f_info;
     Item *item;
     while ((item = it++))
     {
@@ -152,29 +95,28 @@ proc_analyse_init(THD *thd, ORDER *param, select_result *result,
         if (item->type() == Item::FIELD_ITEM &&
             ((Item_field*) item)->field->type() == MYSQL_TYPE_LONGLONG &&
             ((Field_longlong*) ((Item_field*) item)->field)->unsigned_flag)
-          new_field= new field_ulonglong(item, pc);
+          new_field= new field_ulonglong(item, this);
         else
-          new_field= new field_longlong(item, pc);
+          new_field= new field_longlong(item, this);
         break;
       case REAL_RESULT:
-        new_field= new field_real(item, pc);
+        new_field= new field_real(item, this);
         break;
       case DECIMAL_RESULT:
-        new_field= new field_decimal(item, pc);
+        new_field= new field_decimal(item, this);
         break;
       case STRING_RESULT:
-        new_field= new field_str(item, pc);
+        new_field= new field_str(item, this);
         break;
       default:
-        goto err;
+        DBUG_RETURN(true);
       }
-      *f_info++= new_field;
+      if (new_field == NULL)
+        DBUG_RETURN(true);
+      *info++= new_field;
     }
   }
-  DBUG_RETURN(pc);
-err:
-  delete pc;
-  DBUG_RETURN(0);
+  DBUG_RETURN(false);
 }
 
 
@@ -281,33 +223,33 @@ bool get_ev_num_info(EV_NUM_INFO *ev_info, NUM_INFO *info, const char *num)
   {
     if (((longlong) info->ullval) < 0)
       return 0; // Impossible to store as a negative number
-    ev_info->llval =  -(longlong) max((ulonglong) -ev_info->llval, 
-				      info->ullval);
-    ev_info->min_dval = (double) -max(-ev_info->min_dval, info->dval);
+    ev_info->llval =  - max<longlong>((ulonglong) -ev_info->llval, 
+                                           info->ullval);
+    ev_info->min_dval = - max<double>(-ev_info->min_dval, info->dval);
   }
   else		// ulonglong is as big as bigint in MySQL
   {
     if ((check_ulonglong(num, info->integers) == DECIMAL_NUM))
       return 0;
-    ev_info->ullval = (ulonglong) max(ev_info->ullval, info->ullval);
-    ev_info->max_dval =  (double) max(ev_info->max_dval, info->dval);
+    ev_info->ullval = max<ulonglong>(ev_info->ullval, info->ullval);
+    ev_info->max_dval = max<double>(ev_info->max_dval, info->dval);
   }
   return 1;
 } // get_ev_num_info
 
-void free_string(void* key, TREE_FREE action __attribute__((unused)),
-                 void *param __attribute__((unused)))
+void free_string(void* key, TREE_FREE action MY_ATTRIBUTE((unused)),
+                 const void *param MY_ATTRIBUTE((unused)))
 {
-  reinterpret_cast<String*>(key)->free();
+  reinterpret_cast<String*>(key)->mem_free();
 }
 
 void field_str::add()
 {
   char buff[MAX_FIELD_WIDTH], *ptr;
   String s(buff, sizeof(buff),&my_charset_bin), *res;
-  ulong length;
+  size_t length;
 
-  if (!(res = item->val_str(&s)))
+  if (!(res= item->str_result(&s)))
   {
     nulls++;
     return;
@@ -324,12 +266,12 @@ void field_str::add()
 
   if (can_be_still_num)
   {
-    bzero((char*) &num_info, sizeof(num_info));
+    memset(&num_info, 0, sizeof(num_info));
     if (!test_if_number(&num_info, res->ptr(), (uint) length))
       can_be_still_num = 0;
     if (!found)
     {
-      bzero((char*) &ev_num_info, sizeof(ev_num_info));
+      memset(&ev_num_info, 0, sizeof(ev_num_info));
       was_zero_fill = num_info.zerofill;
     }
     else if (num_info.zerofill != was_zero_fill && !was_maybe_zerofill)
@@ -375,7 +317,7 @@ void field_str::add()
       }
       else
       {
-	bzero((char*) &s, sizeof(s));  // Let tree handle free of this
+	memset(static_cast<void*>(&s), 0, sizeof(s));  // Let tree handle free of this
 	if ((treemem += length) > pc->max_treemem)
 	{
 	  room_in_tree = 0;	 // Remove tree, too big tree
@@ -394,8 +336,9 @@ void field_str::add()
 void field_real::add()
 {
   char buff[MAX_FIELD_WIDTH], *ptr, *end;
-  double num= item->val_real();
-  uint length, zero_count, decs;
+  double num= item->val_result();
+  size_t length;
+  uint zero_count, decs;
   TREE_ELEMENT *element;
 
   if (item->null_value)
@@ -414,13 +357,9 @@ void field_real::add()
   }
   else
   {
-#ifdef HAVE_SNPRINTF
     buff[sizeof(buff)-1]=0;			// Safety
-    snprintf(buff, sizeof(buff)-1, "%-.*f", (int) decs, num);
-    length = (uint) strlen(buff);
-#else
-    length= sprintf(buff, "%-.*f", (int) decs, num);
-#endif
+    my_snprintf(buff, sizeof(buff)-1, "%-.*f", (int) decs, num);
+    length= strlen(buff);
 
     // We never need to check further than this
     end = buff + length - 1 - decs + max_notzero_dec_len;
@@ -477,7 +416,7 @@ void field_real::add()
 void field_decimal::add()
 {
   /*TODO - remove rounding stuff after decimal_div returns proper frac */
-  my_decimal dec_buf, *dec= item->val_decimal(&dec_buf);
+  my_decimal dec_buf, *dec= item->val_decimal_result(&dec_buf);
   my_decimal rounded;
   uint length;
   TREE_ELEMENT *element;
@@ -554,8 +493,8 @@ void field_decimal::add()
 void field_longlong::add()
 {
   char buff[MAX_FIELD_WIDTH];
-  longlong num = item->val_int();
-  uint length = (uint) (longlong10_to_str(num, buff, -10) - buff);
+  longlong num= item->val_int_result();
+  uint length= (uint) (longlong10_to_str(num, buff, -10) - buff);
   TREE_ELEMENT *element;
 
   if (item->null_value)
@@ -610,8 +549,8 @@ void field_longlong::add()
 void field_ulonglong::add()
 {
   char buff[MAX_FIELD_WIDTH];
-  longlong num = item->val_int();
-  uint length = (uint) (longlong10_to_str(num, buff, 10) - buff);
+  longlong num= item->val_int_result();
+  uint length= (uint) (longlong10_to_str(num, buff, 10) - buff);
   TREE_ELEMENT *element;
 
   if (item->null_value)
@@ -663,7 +602,7 @@ void field_ulonglong::add()
 } // field_ulonglong::add
 
 
-int analyse::send_row(List<Item> & /* field_list */)
+bool Query_result_analyse::send_data(List<Item> & /* field_list */)
 {
   field_info **f = f_info;
 
@@ -673,11 +612,11 @@ int analyse::send_row(List<Item> & /* field_list */)
   {
     (*f)->add();
   }
-  return 0;
-} // analyse::send_row
+  return false;
+}
 
 
-int analyse::end_of_records()
+bool Query_result_analyse::send_eof()
 {
   field_info **f = f_info;
   char buff[MAX_FIELD_WIDTH];
@@ -685,6 +624,9 @@ int analyse::end_of_records()
 	 s_max(buff, sizeof(buff),&my_charset_bin),
 	 ans(buff, sizeof(buff),&my_charset_bin);
 
+  if (rows == 0) // for backward compatibility
+    goto ok;
+
   for (; f != f_end; f++)
   {
     func_items[0]->set((*f)->item->full_name());
@@ -748,7 +690,7 @@ int analyse::end_of_records()
       output_str_length = tmp_str.length();
       func_items[9]->set(tmp_str.ptr(), tmp_str.length(), tmp_str.charset());
       if (result->send_data(result_fields))
-	return -1;
+	goto error;
       continue;
     }
 
@@ -793,10 +735,14 @@ int analyse::end_of_records()
       ans.append(STRING_WITH_LEN(" NOT NULL"));
     func_items[9]->set(ans.ptr(), ans.length(), ans.charset());
     if (result->send_data(result_fields))
-      return -1;
+      goto error;
   }
-  return 0;
-} // analyse::end_of_records
+ok:
+  return result->send_eof();
+error:
+  abort_result_set();
+  return true;
+} // Query_result_analyse::send_eof
 
 
 void field_str::get_opt_type(String *answer, ha_rows total_rows)
@@ -832,7 +778,7 @@ void field_str::get_opt_type(String *answer, ha_rows total_rows)
       sprintf(buff, "INT(%d)", num_info.integers);
     else
       sprintf(buff, "BIGINT(%d)", num_info.integers);
-    answer->append(buff, (uint) strlen(buff));
+    answer->append(buff, strlen(buff));
     if (ev_num_info.llval >= 0 && ev_num_info.min_dval >= 0)
       answer->append(STRING_WITH_LEN(" UNSIGNED"));
     if (num_info.zerofill)
@@ -850,12 +796,12 @@ void field_str::get_opt_type(String *answer, ha_rows total_rows)
     else if ((max_length * (total_rows - nulls)) < (sum + total_rows))
     {
       sprintf(buff, "CHAR(%d)", (int) max_length);
-      answer->append(buff, (uint) strlen(buff));
+      answer->append(buff, strlen(buff));
     }
     else
     {
       sprintf(buff, "VARCHAR(%d)", (int) max_length);
-      answer->append(buff, (uint) strlen(buff));
+      answer->append(buff, strlen(buff));
     }
   }
   else if (max_length < (1L << 16))
@@ -883,7 +829,7 @@ void field_str::get_opt_type(String *answer, ha_rows total_rows)
 
 
 void field_real::get_opt_type(String *answer,
-			      ha_rows total_rows __attribute__((unused)))
+			      ha_rows total_rows MY_ATTRIBUTE((unused)))
 {
   char buff[MAX_FIELD_WIDTH];
 
@@ -905,7 +851,7 @@ void field_real::get_opt_type(String *answer,
       sprintf(buff, "INT(%d)", len);
     else
       sprintf(buff, "BIGINT(%d)", len);
-    answer->append(buff, (uint) strlen(buff));
+    answer->append(buff, strlen(buff));
     if (min_arg >= 0)
       answer->append(STRING_WITH_LEN(" UNSIGNED"));
   }
@@ -924,7 +870,7 @@ void field_real::get_opt_type(String *answer,
     else
       sprintf(buff, "DOUBLE(%d,%d)", (int) max_length - (item->decimals + 1) + max_notzero_dec_len,
 	      max_notzero_dec_len);
-    answer->append(buff, (uint) strlen(buff));
+    answer->append(buff, strlen(buff));
   }
   // if item is FIELD_ITEM, it _must_be_ Field_num in this class
   if (item->type() == Item::FIELD_ITEM &&
@@ -936,7 +882,7 @@ void field_real::get_opt_type(String *answer,
 
 
 void field_longlong::get_opt_type(String *answer,
-				  ha_rows total_rows __attribute__((unused)))
+				  ha_rows total_rows MY_ATTRIBUTE((unused)))
 {
   char buff[MAX_FIELD_WIDTH];
 
@@ -953,7 +899,7 @@ void field_longlong::get_opt_type(String *answer,
     sprintf(buff, "INT(%d)", (int) max_length);
   else
     sprintf(buff, "BIGINT(%d)", (int) max_length);
-  answer->append(buff, (uint) strlen(buff));
+  answer->append(buff, strlen(buff));
   if (min_arg >= 0)
     answer->append(STRING_WITH_LEN(" UNSIGNED"));
 
@@ -967,7 +913,7 @@ void field_longlong::get_opt_type(String *answer,
 
 
 void field_ulonglong::get_opt_type(String *answer,
-				   ha_rows total_rows __attribute__((unused)))
+				   ha_rows total_rows MY_ATTRIBUTE((unused)))
 {
   char buff[MAX_FIELD_WIDTH];
 
@@ -982,7 +928,7 @@ void field_ulonglong::get_opt_type(String *answer,
   else
     sprintf(buff, "BIGINT(%d) UNSIGNED", (int) max_length);
   // if item is FIELD_ITEM, it _must_be_ Field_num in this class
-  answer->append(buff, (uint) strlen(buff));
+  answer->append(buff, strlen(buff));
   if (item->type() == Item::FIELD_ITEM &&
       // a single number shouldn't be zerofill
       max_length != 1 &&
@@ -992,20 +938,20 @@ void field_ulonglong::get_opt_type(String *answer,
 
 
 void field_decimal::get_opt_type(String *answer,
-                                 ha_rows total_rows __attribute__((unused)))
+                                 ha_rows total_rows MY_ATTRIBUTE((unused)))
 {
   my_decimal zero;
   char buff[MAX_FIELD_WIDTH];
-  uint length;
+  size_t length;
 
   my_decimal_set_zero(&zero);
   my_bool is_unsigned= (my_decimal_cmp(&zero, &min_arg) >= 0);
 
   length= my_snprintf(buff, sizeof(buff), "DECIMAL(%d, %d)",
-                      (int) (max_length - (item->decimals ? 1 : 0)),
-                      item->decimals);
+                      static_cast<int>(max_length - (item->decimals ? 1 : 0)),
+                      static_cast<int>(item->decimals));
   if (is_unsigned)
-    length= (uint) (strmov(buff+length, " UNSIGNED")- buff);
+    length= (my_stpcpy(buff+length, " UNSIGNED")- buff);
   answer->append(buff, length);
 }
 
@@ -1028,7 +974,7 @@ String *field_decimal::avg(String *s, ha_rows rows)
 {
   if (!(rows - nulls))
   {
-    s->set_real((double) 0.0, 1,my_thd_charset);
+    s->set_real(0.0, 1,my_thd_charset);
     return s;
   }
   my_decimal num, avg_val, rounded_avg;
@@ -1049,7 +995,7 @@ String *field_decimal::std(String *s, ha_rows rows)
 {
   if (!(rows - nulls))
   {
-    s->set_real((double) 0.0, 1,my_thd_charset);
+    s->set_real(0.0, 1,my_thd_charset);
     return s;
   }
   my_decimal num, tmp, sum2, sum2d;
@@ -1062,7 +1008,7 @@ String *field_decimal::std(String *s, ha_rows rows)
   my_decimal_sub(E_DEC_FATAL_ERROR, &sum2, sum_sqr+cur_sum, &tmp);
   my_decimal_div(E_DEC_FATAL_ERROR, &tmp, &sum2, &num, prec_increment);
   my_decimal2double(E_DEC_FATAL_ERROR, &tmp, &std_sqr);
-  s->set_real(((double) std_sqr <= 0.0 ? 0.0 : sqrt(std_sqr)),
+  s->set_real((std_sqr <= 0.0 ? 0.0 : sqrt(std_sqr)),
          min(item->decimals + prec_increment, NOT_FIXED_DEC), my_thd_charset);
 
   return s;
@@ -1070,7 +1016,7 @@ String *field_decimal::std(String *s, ha_rows rows)
 
 
 int collect_string(String *element,
-		   element_count count __attribute__((unused)),
+		   element_count count MY_ATTRIBUTE((unused)),
 		   TREE_INFO *info)
 {
   if (info->found)
@@ -1085,7 +1031,7 @@ int collect_string(String *element,
 } // collect_string
 
 
-int collect_real(double *element, element_count count __attribute__((unused)),
+int collect_real(double *element, element_count count MY_ATTRIBUTE((unused)),
 		 TREE_INFO *info)
 {
   char buff[MAX_FIELD_WIDTH];
@@ -1106,7 +1052,7 @@ int collect_real(double *element, element_count count __attribute__((unused)),
 int collect_decimal(uchar *element, element_count count,
                     TREE_INFO *info)
 {
-  char buff[DECIMAL_MAX_STR_LENGTH];
+  char buff[DECIMAL_MAX_STR_LENGTH + 1];
   String s(buff, sizeof(buff),&my_charset_bin);
 
   if (info->found)
@@ -1126,7 +1072,7 @@ int collect_decimal(uchar *element, element_count count,
 
 
 int collect_longlong(longlong *element,
-		     element_count count __attribute__((unused)),
+		     element_count count MY_ATTRIBUTE((unused)),
 		     TREE_INFO *info)
 {
   char buff[MAX_FIELD_WIDTH];
@@ -1145,7 +1091,7 @@ int collect_longlong(longlong *element,
 
 
 int collect_ulonglong(ulonglong *element,
-		      element_count count __attribute__((unused)),
+		      element_count count MY_ATTRIBUTE((unused)),
 		      TREE_INFO *info)
 {
   char buff[MAX_FIELD_WIDTH];
@@ -1163,10 +1109,11 @@ int collect_ulonglong(ulonglong *element,
 } // collect_ulonglong
 
 
-bool analyse::change_columns(List<Item> &field_list)
+/**
+  Create items for substituted output columns (both metadata and data)
+*/
+bool Query_result_analyse::change_columns()
 {
-  field_list.empty();
-
   func_items[0] = new Item_proc_string("Field_name", 255);
   func_items[1] = new Item_proc_string("Min_value", 255);
   func_items[1]->maybe_null = 1;
@@ -1180,13 +1127,45 @@ bool analyse::change_columns(List<Item> &field_list)
   func_items[8] = new Item_proc_string("Std", 255);
   func_items[8]->maybe_null = 1;
   func_items[9] = new Item_proc_string("Optimal_fieldtype",
-				       max(64, output_str_length));
-
+				       max<size_t>(64U, output_str_length));
+  result_fields.empty();
   for (uint i = 0; i < array_elements(func_items); i++)
-    field_list.push_back(func_items[i]);
-  result_fields = field_list;
-  return 0;
-} // analyse::change_columns
+  {
+    if (func_items[i] == NULL)
+      return true;
+    result_fields.push_back(func_items[i]);
+  }
+  return false;
+} // Query_result_analyse::change_columns
+
+
+void Query_result_analyse::cleanup()
+{
+  if (f_info)
+  {
+    for (field_info **f= f_info; f != f_end; f++)
+      delete (*f);
+    f_info= f_end= NULL;
+  }
+  rows= 0;
+  output_str_length= 0;
+}
+
+
+bool Query_result_analyse::send_result_set_metadata(List<Item> &fields,
+                                                    uint flag)
+{
+  return (init(fields) || change_columns() ||
+	  result->send_result_set_metadata(result_fields, flag));
+}
+
+
+void Query_result_analyse::abort_result_set()
+{
+  cleanup();
+  return result->abort_result_set();
+}
+
 
 int compare_double(const double *s, const double *t)
 {
@@ -1260,7 +1239,7 @@ bool append_escaped(String *to_str, String *from_str)
 {
   char *from, *end, c;
 
-  if (to_str->realloc(to_str->length() + from_str->length()))
+  if (to_str->mem_realloc(to_str->length() + from_str->length()))
     return 1;
 
   from= (char*) from_str->ptr();
@@ -1290,3 +1269,4 @@ bool append_escaped(String *to_str, String *from_str)
   }
   return 0;
 }
+
