@@ -664,7 +664,7 @@ bool Sql_cmd_delete::mysql_prepare_delete(THD *thd)
     List<Item>   fields;
     List<Item>   all_fields;
 
-    memset(&tables, 0, sizeof(tables));
+    memset(static_cast<void*>(&tables), 0, sizeof(tables));
     tables.table = table_list->table;
     tables.alias = table_list->alias;
 
