@@ -68,7 +68,7 @@
 
 
 /* Copy the first part of user declarations.  */
-#line 21 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:339  */
+#line 21 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:339  */
 
 #include "sql_class.h"
 #include "parse_tree_hints.h"
@@ -77,7 +77,7 @@
 
 #define NEW_PTN new (thd->mem_root)
 
-#line 81 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:339  */
+#line 81 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:339  */
 
 # ifndef YY_NULLPTR
 #  if defined __cplusplus && 201103L <= __cplusplus
@@ -97,8 +97,8 @@
 
 /* In a future release of Bison, this section will be replaced
    by #include "sql_hints.yy.h".  */
-#ifndef YY_HINT_PARSER_VAR_LIB_JENKINS_JOBS_PERCONA_SERVER_5_7_SOURCE_TARBALLS_ROCKS_WORKSPACE_SQL_SQL_HINTS_YY_H_INCLUDED
-# define YY_HINT_PARSER_VAR_LIB_JENKINS_JOBS_PERCONA_SERVER_5_7_SOURCE_TARBALLS_ROCKS_WORKSPACE_SQL_SQL_HINTS_YY_H_INCLUDED
+#ifndef YY_HINT_PARSER_MNT_WORKSPACE_PERCONA_SERVER_5_7_SOURCE_TARBALLS_ROCKS_SQL_SQL_HINTS_YY_H_INCLUDED
+# define YY_HINT_PARSER_MNT_WORKSPACE_PERCONA_SERVER_5_7_SOURCE_TARBALLS_ROCKS_SQL_SQL_HINTS_YY_H_INCLUDED
 /* Debug traces.  */
 #ifndef YYDEBUG
 # define YYDEBUG 0
@@ -168,11 +168,11 @@ extern int HINT_PARSER_debug;
 
 int HINT_PARSER_parse (class THD *thd, class Hint_scanner *scanner, class PT_hint_list **ret);
 
-#endif /* !YY_HINT_PARSER_VAR_LIB_JENKINS_JOBS_PERCONA_SERVER_5_7_SOURCE_TARBALLS_ROCKS_WORKSPACE_SQL_SQL_HINTS_YY_H_INCLUDED  */
+#endif /* !YY_HINT_PARSER_MNT_WORKSPACE_PERCONA_SERVER_5_7_SOURCE_TARBALLS_ROCKS_SQL_SQL_HINTS_YY_H_INCLUDED  */
 
 /* Copy the second part of user declarations.  */
 
-#line 176 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:358  */
+#line 176 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:358  */
 
 #ifdef short
 # undef short
@@ -1331,44 +1331,44 @@ yyreduce:
   switch (yyn)
     {
         case 2:
-#line 119 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 119 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     { *ret= (yyvsp[-1].hint_list); }
-#line 1337 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1337 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 3:
-#line 121 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 121 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     { *ret= (yyvsp[-2].hint_list); }
-#line 1343 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1343 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 4:
-#line 123 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 123 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     { *ret= NULL; }
-#line 1349 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1349 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 5:
-#line 128 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 128 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint_list)= NEW_PTN PT_hint_list(thd->mem_root);
             if ((yyval.hint_list) == NULL || (yyval.hint_list)->push_back((yyvsp[0].hint)))
               YYABORT; // OOM
           }
-#line 1359 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1359 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 6:
-#line 134 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 134 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyvsp[-1].hint_list)->push_back((yyvsp[0].hint));
             (yyval.hint_list)= (yyvsp[-1].hint_list);
           }
-#line 1368 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1368 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 12:
-#line 151 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 151 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             int error;
             char *end= const_cast<char *>((yyvsp[-1].hint_string).str + (yyvsp[-1].hint_string).length);
@@ -1386,345 +1386,345 @@ yyreduce:
                 YYABORT; // OOM
             }
           }
-#line 1390 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1390 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 13:
-#line 172 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 172 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     { (yyval.hint_param_table_list).init(thd->mem_root); }
-#line 1396 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1396 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 15:
-#line 178 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 178 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint_param_table_list).init(thd->mem_root);
             if ((yyval.hint_param_table_list).push_back((yyvsp[0].hint_param_table)))
               YYABORT; // OOM
           }
-#line 1406 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1406 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 16:
-#line 184 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 184 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             if ((yyvsp[-2].hint_param_table_list).push_back((yyvsp[0].hint_param_table)))
               YYABORT; // OOM
             (yyval.hint_param_table_list)= (yyvsp[-2].hint_param_table_list);
           }
-#line 1416 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1416 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 17:
-#line 192 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 192 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     { (yyval.hint_param_table_list).init(thd->mem_root); }
-#line 1422 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1422 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 19:
-#line 198 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 198 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint_param_table_list).init(thd->mem_root);
             if ((yyval.hint_param_table_list).push_back((yyvsp[0].hint_param_table)))
               YYABORT; // OOM
           }
-#line 1432 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1432 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 20:
-#line 204 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 204 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             if ((yyvsp[-2].hint_param_table_list).push_back((yyvsp[0].hint_param_table)))
               YYABORT; // OOM
             (yyval.hint_param_table_list)= (yyvsp[-2].hint_param_table_list);
           }
-#line 1442 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1442 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 21:
-#line 212 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 212 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     { (yyval.hint_param_index_list).init(thd->mem_root); }
-#line 1448 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1448 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 23:
-#line 218 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 218 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint_param_index_list).init(thd->mem_root);
             if ((yyval.hint_param_index_list).push_back((yyvsp[0].hint_string)))
               YYABORT; // OOM
           }
-#line 1458 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1458 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 24:
-#line 224 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 224 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             if ((yyvsp[-2].hint_param_index_list).push_back((yyvsp[0].hint_string)))
               YYABORT; // OOM
             (yyval.hint_param_index_list)= (yyvsp[-2].hint_param_index_list);
           }
-#line 1468 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1468 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 26:
-#line 237 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 237 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint_param_table).table= (yyvsp[0].hint_string);
             (yyval.hint_param_table).opt_query_block= NULL_CSTR;
           }
-#line 1477 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1477 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 27:
-#line 245 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 245 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint_param_table).table= (yyvsp[-1].hint_string);
             (yyval.hint_param_table).opt_query_block= (yyvsp[0].hint_string);
           }
-#line 1486 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1486 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 29:
-#line 254 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 254 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint_param_table).table= (yyvsp[0].hint_string);
             (yyval.hint_param_table).opt_query_block= (yyvsp[-1].hint_string);
           }
-#line 1495 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1495 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 30:
-#line 261 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 261 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     { (yyval.hint_string)= NULL_CSTR; }
-#line 1501 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1501 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 32:
-#line 267 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 267 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint)= NEW_PTN PT_qb_level_hint((yyvsp[-2].hint_string), TRUE, SEMIJOIN_HINT_ENUM, (yyvsp[-1].ulong_num));
             if ((yyval.hint) == NULL)
               YYABORT; // OOM
           }
-#line 1511 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1511 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 33:
-#line 274 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 274 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint)= NEW_PTN PT_qb_level_hint((yyvsp[-2].hint_string), FALSE, SEMIJOIN_HINT_ENUM, (yyvsp[-1].ulong_num));
             if ((yyval.hint) == NULL)
               YYABORT; // OOM
           }
-#line 1521 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1521 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 34:
-#line 281 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 281 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint)= NEW_PTN PT_qb_level_hint((yyvsp[-2].hint_string), TRUE, SUBQUERY_HINT_ENUM, (yyvsp[-1].ulong_num));
             if ((yyval.hint) == NULL)
               YYABORT; // OOM
           }
-#line 1531 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1531 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 35:
-#line 289 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 289 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     { (yyval.ulong_num)= 0; }
-#line 1537 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1537 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 36:
-#line 291 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 291 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.ulong_num)= (yyvsp[0].ulong_num);
           }
-#line 1545 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1545 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 37:
-#line 295 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 295 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.ulong_num)= (yyvsp[-2].ulong_num) | (yyvsp[0].ulong_num);
           }
-#line 1553 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1553 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 38:
-#line 301 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 301 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     { (yyval.ulong_num)= OPTIMIZER_SWITCH_FIRSTMATCH; }
-#line 1559 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1559 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 39:
-#line 302 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 302 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     { (yyval.ulong_num)= OPTIMIZER_SWITCH_LOOSE_SCAN; }
-#line 1565 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1565 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 40:
-#line 303 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 303 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     { (yyval.ulong_num)= OPTIMIZER_SWITCH_MATERIALIZATION; }
-#line 1571 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1571 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 41:
-#line 304 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 304 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     { (yyval.ulong_num)= OPTIMIZER_SWITCH_DUPSWEEDOUT; }
-#line 1577 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1577 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 42:
-#line 308 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 308 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     { (yyval.ulong_num)=
                                    Item_exists_subselect::EXEC_MATERIALIZATION; }
-#line 1584 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1584 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 43:
-#line 310 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 310 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     { (yyval.ulong_num)= Item_exists_subselect::EXEC_EXISTS; }
-#line 1590 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1590 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 44:
-#line 316 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 316 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint)= NEW_PTN PT_table_level_hint(NULL_CSTR, (yyvsp[-1].hint_param_table_list), TRUE, (yyvsp[-3].hint_type));
             if ((yyval.hint) == NULL)
               YYABORT; // OOM
           }
-#line 1600 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1600 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 45:
-#line 323 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 323 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint)= NEW_PTN PT_table_level_hint((yyvsp[-2].hint_string), (yyvsp[-1].hint_param_table_list), TRUE, (yyvsp[-4].hint_type));
             if ((yyval.hint) == NULL)
               YYABORT; // OOM
           }
-#line 1610 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1610 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 46:
-#line 329 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 329 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint)= NEW_PTN PT_table_level_hint(NULL_CSTR, (yyvsp[-1].hint_param_table_list), FALSE, (yyvsp[-3].hint_type));
             if ((yyval.hint) == NULL)
               YYABORT; // OOM
           }
-#line 1620 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1620 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 47:
-#line 336 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 336 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint)= NEW_PTN PT_table_level_hint((yyvsp[-2].hint_string), (yyvsp[-1].hint_param_table_list), FALSE, (yyvsp[-4].hint_type));
             if ((yyval.hint) == NULL)
               YYABORT; // OOM
           }
-#line 1630 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1630 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 48:
-#line 346 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 346 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint)= NEW_PTN PT_key_level_hint((yyvsp[-2].hint_param_table), (yyvsp[-1].hint_param_index_list), TRUE, (yyvsp[-4].hint_type));
             if ((yyval.hint) == NULL)
               YYABORT; // OOM
           }
-#line 1640 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1640 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 49:
-#line 353 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 353 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint)= NEW_PTN PT_key_level_hint((yyvsp[-2].hint_param_table), (yyvsp[-1].hint_param_index_list), FALSE, (yyvsp[-4].hint_type));
             if ((yyval.hint) == NULL)
               YYABORT; // OOM
           }
-#line 1650 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1650 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 50:
-#line 362 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 362 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint_type)= BKA_HINT_ENUM;
           }
-#line 1658 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1658 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 51:
-#line 366 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 366 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint_type)= BNL_HINT_ENUM;
           }
-#line 1666 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1666 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 52:
-#line 373 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 373 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint_type)= BKA_HINT_ENUM;
           }
-#line 1674 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1674 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 53:
-#line 377 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 377 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint_type)= BNL_HINT_ENUM;
           }
-#line 1682 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1682 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 54:
-#line 384 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 384 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint_type)= MRR_HINT_ENUM;
           }
-#line 1690 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1690 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 55:
-#line 388 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 388 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint_type)= NO_RANGE_HINT_ENUM;
           }
-#line 1698 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1698 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 56:
-#line 395 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 395 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint_type)= ICP_HINT_ENUM;
           }
-#line 1706 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1706 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 57:
-#line 399 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 399 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint_type)= MRR_HINT_ENUM;
           }
-#line 1714 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1714 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 58:
-#line 406 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 406 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint)= NEW_PTN PT_hint_qb_name((yyvsp[-1].hint_string));
             if ((yyval.hint) == NULL)
               YYABORT; // OOM
           }
-#line 1724 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1724 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
 
-#line 1728 "/var/lib/jenkins/jobs/percona-server-5.7-source-tarballs-rocks/workspace/sql/sql_hints.yy.cc" /* yacc.c:1646  */
+#line 1728 "/mnt/workspace/percona-server-5.7-source-tarballs-rocks/sql/sql_hints.yy.cc" /* yacc.c:1646  */
       default: break;
     }
   /* User semantic actions sometimes alter yychar, and that requires
