@@ -30,8 +30,8 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
-#ifndef YY_HINT_PARSER_VAR_LIB_JENKINS_JOBS_PERCONA_SERVER_5_7_SOURCE_TARBALLS_ROCKS_WORKSPACE_SQL_SQL_HINTS_YY_H_INCLUDED
-# define YY_HINT_PARSER_VAR_LIB_JENKINS_JOBS_PERCONA_SERVER_5_7_SOURCE_TARBALLS_ROCKS_WORKSPACE_SQL_SQL_HINTS_YY_H_INCLUDED
+#ifndef YY_HINT_PARSER_MNT_WORKSPACE_PERCONA_SERVER_5_7_SOURCE_TARBALLS_ROCKS_SQL_SQL_HINTS_YY_H_INCLUDED
+# define YY_HINT_PARSER_MNT_WORKSPACE_PERCONA_SERVER_5_7_SOURCE_TARBALLS_ROCKS_SQL_SQL_HINTS_YY_H_INCLUDED
 /* Debug traces.  */
 #ifndef YYDEBUG
 # define YYDEBUG 0
@@ -101,4 +101,4 @@ extern int HINT_PARSER_debug;
 
 int HINT_PARSER_parse (class THD *thd, class Hint_scanner *scanner, class PT_hint_list **ret);
 
-#endif /* !YY_HINT_PARSER_VAR_LIB_JENKINS_JOBS_PERCONA_SERVER_5_7_SOURCE_TARBALLS_ROCKS_WORKSPACE_SQL_SQL_HINTS_YY_H_INCLUDED  */
+#endif /* !YY_HINT_PARSER_MNT_WORKSPACE_PERCONA_SERVER_5_7_SOURCE_TARBALLS_ROCKS_SQL_SQL_HINTS_YY_H_INCLUDED  */
