@@ -30,8 +30,8 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
-#ifndef YY_MYSQL_VAR_LIB_JENKINS_JOBS_PERCONA_SERVER_5_7_SOURCE_TARBALLS_ROCKS_WORKSPACE_SQL_SQL_YACC_H_INCLUDED
-# define YY_MYSQL_VAR_LIB_JENKINS_JOBS_PERCONA_SERVER_5_7_SOURCE_TARBALLS_ROCKS_WORKSPACE_SQL_SQL_YACC_H_INCLUDED
+#ifndef YY_MYSQL_MNT_WORKSPACE_PERCONA_SERVER_5_7_SOURCE_TARBALLS_ROCKS_SQL_SQL_YACC_H_INCLUDED
+# define YY_MYSQL_MNT_WORKSPACE_PERCONA_SERVER_5_7_SOURCE_TARBALLS_ROCKS_SQL_SQL_YACC_H_INCLUDED
 /* Debug traces.  */
 #ifndef YYDEBUG
 # define YYDEBUG 0
@@ -711,7 +711,9 @@ extern int MYSQLdebug;
     XOR = 921,
     YEAR_MONTH_SYM = 922,
     YEAR_SYM = 923,
-    ZEROFILL = 924
+    ZEROFILL = 924,
+    JSON_OBJECTAGG = 925,
+    JSON_ARRAYAGG = 926
   };
 #endif
 /* Tokens.  */
@@ -1382,6 +1384,8 @@ extern int MYSQLdebug;
 #define YEAR_MONTH_SYM 922
 #define YEAR_SYM 923
 #define ZEROFILL 924
+#define JSON_OBJECTAGG 925
+#define JSON_ARRAYAGG 926
 
 /* Value type.  */
 
@@ -1403,4 +1407,4 @@ struct YYLTYPE
 
 int MYSQLparse (class THD *YYTHD);
 
-#endif /* !YY_MYSQL_VAR_LIB_JENKINS_JOBS_PERCONA_SERVER_5_7_SOURCE_TARBALLS_ROCKS_WORKSPACE_SQL_SQL_YACC_H_INCLUDED  */
+#endif /* !YY_MYSQL_MNT_WORKSPACE_PERCONA_SERVER_5_7_SOURCE_TARBALLS_ROCKS_SQL_SQL_YACC_H_INCLUDED  */
