@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1996, 2016, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1996, 2018, Oracle and/or its affiliates. All Rights Reserved.
 Copyright (c) 2012, Facebook Inc.
 
 This program is free software; you can redistribute it and/or modify it under
@@ -1137,6 +1137,12 @@ dict_index_add_to_cache(
 	ibool		strict)
 	MY_ATTRIBUTE((warn_unused_result));
 
+/** Clears the virtual column's index list before index is being freed.
+@param[in]  index   Index being freed */
+void
+dict_index_remove_from_v_col_list(
+	dict_index_t* index);
+
 /** Adds an index to the dictionary cache, with possible indexing newly
 added column.
 @param[in]	table	table on which the index is
@@ -2115,6 +2121,13 @@ bool
 dict_table_have_virtual_index(
 	dict_table_t*	table);
 
+/** Allocate memory for intrinsic cache elements in the index
+ * @param[in]      index   index object */
+UNIV_INLINE
+void
+dict_allocate_mem_intrinsic_cache(
+                dict_index_t*           index);
+
 #endif /* !UNIV_HOTBACKUP */
 /*************************************************************************
 set is_corrupt flag by space_id*/
