@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1994, 2016, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1994, 2017, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -241,6 +241,15 @@ purposes.
 ulint
 ut_time_ms(void);
 /*============*/
+#ifdef _WIN32
+/**********************************************************//**
+Initialise highest available time resolution API on Windows
+@return 0 if all OK else -1 */
+int
+ut_win_init_time();
+
+#endif /* _WIN32 */
+
 #endif /* !UNIV_HOTBACKUP */
 
 /**********************************************************//**
@@ -614,6 +623,19 @@ private:
 	const bool	m_fatal;
 };
 
+/** Emit a warning message if the given predicate is true, otherwise emit an
+informational message. */
+class warn_or_info : public logger {
+public:
+	warn_or_info(bool	pred)
+	: m_warn(pred)
+	{}
+
+	~warn_or_info();
+private:
+	const bool	m_warn;
+};
+
 } // namespace ib
 
 #ifndef UNIV_NONINL
