@@ -1231,6 +1231,19 @@ srv_open_tmp_tablespace(
 		it stays open until shutdown. */
 		if (fil_space_open(tmp_space->name())) {
 
+			if (srv_tmp_tablespace_encrypt) {
+
+				fil_space_t*	space =
+					fil_space_get(temp_space_id);
+				space->flags |= FSP_FLAGS_MASK_ENCRYPTION;
+
+				err = fil_set_encryption(space->id,
+							 Encryption::AES,
+							 NULL,
+							 NULL);
+				ut_a(err == DB_SUCCESS);
+			}
+
 			/* Initialize the header page */
 			mtr_start(&mtr);
 			mtr_set_log_mode(&mtr, MTR_LOG_NO_REDO);
@@ -1867,7 +1880,8 @@ innobase_start_or_create_for_mysql(void)
 		<< size << unit << ", instances = " << srv_buf_pool_instances
 		<< ", chunk size = " << chunk_size << chunk_unit;
 
-	err = buf_pool_init(srv_buf_pool_size, srv_buf_pool_instances);
+	err = buf_pool_init(srv_buf_pool_size, static_cast<bool>(srv_numa_interleave),
+	                    srv_buf_pool_instances);
 
 	if (err != DB_SUCCESS) {
 		ib::error() << "Cannot allocate memory for the buffer pool";
@@ -2688,6 +2702,9 @@ files_checked:
 		srv_start_state_set(SRV_START_STATE_MASTER);
 	}
 
+	/* Enable row log encryption if it is set */
+	log_tmp_enable_encryption_if_set();
+
 	if (!srv_read_only_mode
 	    && srv_force_recovery < SRV_FORCE_NO_BACKGROUND) {
 
