@@ -15,6 +15,8 @@
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 /* C++ standard header files */
+#include <algorithm>
+#include <cctype>
 #include <map>
 #include <string>
 #include <vector>
@@ -71,7 +73,18 @@ static int rdb_i_s_cfstats_fill_table(my_core::THD *const thd,
                                       MY_ATTRIBUTE((__unused__))) {
   DBUG_ENTER_FUNC();
 
-  bool ret;
+  DBUG_ASSERT(tables != nullptr);
+  DBUG_ASSERT(tables->table != nullptr);
+  DBUG_ASSERT(tables->table->field != nullptr);
+
+  int ret = 0;
+  Rdb_hton_init_state::Scoped_lock state_lock(*rdb_get_hton_init_state(),
+                                              false);
+  if (!rdb_get_hton_init_state()->initialized()) {
+    ret = ER_PLUGIN_IS_NOT_LOADED;
+    my_error(ret, MYF(0), rocksdb_hton_name);
+    DBUG_RETURN(ret);
+  }
   uint64_t val;
 
   const std::vector<std::pair<const std::string, std::string>> cf_properties = {
@@ -93,26 +106,24 @@ static int rdb_i_s_cfstats_fill_table(my_core::THD *const thd,
       {rocksdb::DB::Properties::kNumLiveVersions, "NUM_LIVE_VERSIONS"}};
 
   rocksdb::DB *const rdb = rdb_get_rocksdb_db();
+
+  if (!rdb) {
+    DBUG_RETURN(ret);
+  }
+
   const Rdb_cf_manager &cf_manager = rdb_get_cf_manager();
-  DBUG_ASSERT(rdb != nullptr);
 
   for (const auto &cf_name : cf_manager.get_cf_names()) {
-    rocksdb::ColumnFamilyHandle *cfh;
-    bool is_automatic;
-
-    /*
-      Only the cf name is important. Whether it was generated automatically
-      does not matter, so is_automatic is ignored.
-    */
-    cfh = cf_manager.get_cf(cf_name.c_str(), "", nullptr, &is_automatic);
-    if (cfh == nullptr)
+    DBUG_ASSERT(!cf_name.empty());
+    rocksdb::ColumnFamilyHandle *cfh = cf_manager.get_cf(cf_name);
+    if (cfh == nullptr) {
       continue;
+    }
 
     for (const auto &property : cf_properties) {
-      if (!rdb->GetIntProperty(cfh, property.first, &val))
+      if (!rdb->GetIntProperty(cfh, property.first, &val)) {
         continue;
-
-      DBUG_ASSERT(tables != nullptr);
+      }
 
       tables->table->field[RDB_CFSTATS_FIELD::CF_NAME]->store(
           cf_name.c_str(), cf_name.size(), system_charset_info);
@@ -120,12 +131,15 @@ static int rdb_i_s_cfstats_fill_table(my_core::THD *const thd,
           property.second.c_str(), property.second.size(), system_charset_info);
       tables->table->field[RDB_CFSTATS_FIELD::VALUE]->store(val, true);
 
-      ret = my_core::schema_table_store_record(thd, tables->table);
+      ret = static_cast<int>(
+          my_core::schema_table_store_record(thd, tables->table));
 
-      if (ret)
+      if (ret) {
         DBUG_RETURN(ret);
+      }
     }
   }
+
   DBUG_RETURN(0);
 }
 
@@ -162,7 +176,18 @@ static int rdb_i_s_dbstats_fill_table(my_core::THD *const thd,
                                       MY_ATTRIBUTE((__unused__))) {
   DBUG_ENTER_FUNC();
 
-  bool ret;
+  DBUG_ASSERT(tables != nullptr);
+  DBUG_ASSERT(tables->table != nullptr);
+  DBUG_ASSERT(tables->table->field != nullptr);
+
+  int ret = 0;
+  Rdb_hton_init_state::Scoped_lock state_lock(*rdb_get_hton_init_state(),
+                                              false);
+  if (!rdb_get_hton_init_state()->initialized()) {
+    ret = ER_PLUGIN_IS_NOT_LOADED;
+    my_error(ret, MYF(0), rocksdb_hton_name);
+    DBUG_RETURN(ret);
+  }
   uint64_t val;
 
   const std::vector<std::pair<std::string, std::string>> db_properties = {
@@ -172,23 +197,29 @@ static int rdb_i_s_dbstats_fill_table(my_core::THD *const thd,
        "DB_OLDEST_SNAPSHOT_TIME"}};
 
   rocksdb::DB *const rdb = rdb_get_rocksdb_db();
+
+  if (!rdb) {
+    DBUG_RETURN(ret);
+  }
+
   const rocksdb::BlockBasedTableOptions &table_options =
       rdb_get_table_options();
 
   for (const auto &property : db_properties) {
-    if (!rdb->GetIntProperty(property.first, &val))
+    if (!rdb->GetIntProperty(property.first, &val)) {
       continue;
-
-    DBUG_ASSERT(tables != nullptr);
+    }
 
     tables->table->field[RDB_DBSTATS_FIELD::STAT_TYPE]->store(
         property.second.c_str(), property.second.size(), system_charset_info);
     tables->table->field[RDB_DBSTATS_FIELD::VALUE]->store(val, true);
 
-    ret = my_core::schema_table_store_record(thd, tables->table);
+    ret = static_cast<int>(
+        my_core::schema_table_store_record(thd, tables->table));
 
-    if (ret)
+    if (ret) {
       DBUG_RETURN(ret);
+    }
   }
 
   /*
@@ -202,11 +233,13 @@ static int rdb_i_s_dbstats_fill_table(my_core::THD *const thd,
     information from the column family.
    */
   val = (table_options.block_cache ? table_options.block_cache->GetUsage() : 0);
+
   tables->table->field[RDB_DBSTATS_FIELD::STAT_TYPE]->store(
       STRING_WITH_LEN("DB_BLOCK_CACHE_USAGE"), system_charset_info);
   tables->table->field[RDB_DBSTATS_FIELD::VALUE]->store(val, true);
 
-  ret = my_core::schema_table_store_record(thd, tables->table);
+  ret =
+      static_cast<int>(my_core::schema_table_store_record(thd, tables->table));
 
   DBUG_RETURN(ret);
 }
@@ -250,17 +283,36 @@ static int rdb_i_s_perf_context_fill_table(my_core::THD *const thd,
 
   DBUG_ASSERT(thd != nullptr);
   DBUG_ASSERT(tables != nullptr);
+  DBUG_ASSERT(tables->table != nullptr);
 
   int ret = 0;
+  Rdb_hton_init_state::Scoped_lock state_lock(*rdb_get_hton_init_state(),
+                                              false);
+  if (!rdb_get_hton_init_state()->initialized()) {
+    ret = ER_PLUGIN_IS_NOT_LOADED;
+    my_error(ret, MYF(0), rocksdb_hton_name);
+    DBUG_RETURN(ret);
+  }
+
   Field **field = tables->table->field;
+  DBUG_ASSERT(field != nullptr);
+
+  rocksdb::DB *const rdb = rdb_get_rocksdb_db();
+
+  if (!rdb) {
+    DBUG_RETURN(ret);
+  }
 
   const std::vector<std::string> tablenames = rdb_get_open_table_names();
+
   for (const auto &it : tablenames) {
     std::string str, dbname, tablename, partname;
     Rdb_perf_counters counters;
 
-    if (rdb_normalize_tablename(it, &str)) {
-      DBUG_RETURN(HA_ERR_INTERNAL_ERROR);
+    int rc = rdb_normalize_tablename(it, &str);
+
+    if (rc != HA_EXIT_SUCCESS) {
+      DBUG_RETURN(rc);
     }
 
     if (rdb_split_normalized_tablename(str, &dbname, &tablename, &partname)) {
@@ -271,12 +323,11 @@ static int rdb_i_s_perf_context_fill_table(my_core::THD *const thd,
       continue;
     }
 
-    DBUG_ASSERT(field != nullptr);
-
     field[RDB_PERF_CONTEXT_FIELD::TABLE_SCHEMA]->store(
         dbname.c_str(), dbname.size(), system_charset_info);
     field[RDB_PERF_CONTEXT_FIELD::TABLE_NAME]->store(
         tablename.c_str(), tablename.size(), system_charset_info);
+
     if (partname.size() == 0) {
       field[RDB_PERF_CONTEXT_FIELD::PARTITION_NAME]->set_null();
     } else {
@@ -291,9 +342,12 @@ static int rdb_i_s_perf_context_fill_table(my_core::THD *const thd,
           system_charset_info);
       field[RDB_PERF_CONTEXT_FIELD::VALUE]->store(counters.m_value[i], true);
 
-      ret = my_core::schema_table_store_record(thd, tables->table);
-      if (ret)
+      ret = static_cast<int>(
+          my_core::schema_table_store_record(thd, tables->table));
+
+      if (ret) {
         DBUG_RETURN(ret);
+      }
     }
   }
 
@@ -334,26 +388,41 @@ static int rdb_i_s_perf_context_global_fill_table(
 
   DBUG_ASSERT(thd != nullptr);
   DBUG_ASSERT(tables != nullptr);
+  DBUG_ASSERT(tables->table != nullptr);
+  DBUG_ASSERT(tables->table->field != nullptr);
 
   int ret = 0;
+  Rdb_hton_init_state::Scoped_lock state_lock(*rdb_get_hton_init_state(),
+                                              false);
+  if (!rdb_get_hton_init_state()->initialized()) {
+    ret = ER_PLUGIN_IS_NOT_LOADED;
+    my_error(ret, MYF(0), rocksdb_hton_name);
+    DBUG_RETURN(ret);
+  }
+
+  rocksdb::DB *const rdb = rdb_get_rocksdb_db();
+
+  if (!rdb) {
+    DBUG_RETURN(ret);
+  }
 
   // Get a copy of the global perf counters.
   Rdb_perf_counters global_counters;
   rdb_get_global_perf_counters(&global_counters);
 
   for (int i = 0; i < PC_MAX_IDX; i++) {
-    DBUG_ASSERT(tables->table != nullptr);
-    DBUG_ASSERT(tables->table->field != nullptr);
-
     tables->table->field[RDB_PERF_CONTEXT_GLOBAL_FIELD::STAT_TYPE]->store(
         rdb_pc_stat_types[i].c_str(), rdb_pc_stat_types[i].size(),
         system_charset_info);
     tables->table->field[RDB_PERF_CONTEXT_GLOBAL_FIELD::VALUE]->store(
         global_counters.m_value[i], true);
 
-    ret = my_core::schema_table_store_record(thd, tables->table);
-    if (ret)
+    ret = static_cast<int>(
+        my_core::schema_table_store_record(thd, tables->table));
+
+    if (ret) {
       DBUG_RETURN(ret);
+    }
   }
 
   DBUG_RETURN(0);
@@ -396,13 +465,28 @@ static int rdb_i_s_cfoptions_fill_table(my_core::THD *const thd,
   DBUG_ASSERT(thd != nullptr);
   DBUG_ASSERT(tables != nullptr);
 
-  bool ret;
+  int ret = 0;
+  Rdb_hton_init_state::Scoped_lock state_lock(*rdb_get_hton_init_state(),
+                                              false);
+  if (!rdb_get_hton_init_state()->initialized()) {
+    ret = ER_PLUGIN_IS_NOT_LOADED;
+    my_error(ret, MYF(0), rocksdb_hton_name);
+    DBUG_RETURN(ret);
+  }
+
+  rocksdb::DB *const rdb = rdb_get_rocksdb_db();
+
+  if (!rdb) {
+    DBUG_RETURN(ret);
+  }
 
   Rdb_cf_manager &cf_manager = rdb_get_cf_manager();
 
   for (const auto &cf_name : cf_manager.get_cf_names()) {
     std::string val;
     rocksdb::ColumnFamilyOptions opts;
+
+    DBUG_ASSERT(!cf_name.empty());
     cf_manager.get_cf_options(cf_name, &opts);
 
     std::vector<std::pair<std::string, std::string>> cf_option_types = {
@@ -472,29 +556,37 @@ static int rdb_i_s_cfoptions_fill_table(my_core::THD *const thd,
 
     // get MAX_BYTES_FOR_LEVEL_MULTIPLIER_ADDITIONAL option value
     val = opts.max_bytes_for_level_multiplier_additional.empty() ? "NULL" : "";
+
     for (const auto &level : opts.max_bytes_for_level_multiplier_additional) {
       val.append(std::to_string(level) + ":");
     }
+
     val.pop_back();
     cf_option_types.push_back(
         {"MAX_BYTES_FOR_LEVEL_MULTIPLIER_ADDITIONAL", val});
 
     // get COMPRESSION_TYPE option value
     GetStringFromCompressionType(&val, opts.compression);
+
     if (val.empty()) {
       val = "NULL";
     }
+
     cf_option_types.push_back({"COMPRESSION_TYPE", val});
 
     // get COMPRESSION_PER_LEVEL option value
     val = opts.compression_per_level.empty() ? "NULL" : "";
+
     for (const auto &compression_type : opts.compression_per_level) {
       std::string res;
+
       GetStringFromCompressionType(&res, compression_type);
+
       if (!res.empty()) {
         val.append(res + ":");
       }
     }
+
     val.pop_back();
     cf_option_types.push_back({"COMPRESSION_PER_LEVEL", val});
 
@@ -502,12 +594,15 @@ static int rdb_i_s_cfoptions_fill_table(my_core::THD *const thd,
     val = std::to_string(opts.compression_opts.window_bits) + ":";
     val.append(std::to_string(opts.compression_opts.level) + ":");
     val.append(std::to_string(opts.compression_opts.strategy));
+
     cf_option_types.push_back({"COMPRESSION_OPTS", val});
 
     // bottommost_compression
     if (opts.bottommost_compression) {
       std::string res;
+
       GetStringFromCompressionType(&res, opts.bottommost_compression);
+
       if (!res.empty()) {
         cf_option_types.push_back({"BOTTOMMOST_COMPRESSION", res});
       }
@@ -536,12 +631,15 @@ static int rdb_i_s_cfoptions_fill_table(my_core::THD *const thd,
     default:
       val = "NULL";
     }
+
     cf_option_types.push_back({"COMPACTION_STYLE", val});
 
     // get COMPACTION_OPTIONS_UNIVERSAL related options
     const rocksdb::CompactionOptionsUniversal compac_opts =
         opts.compaction_options_universal;
+
     val = "{SIZE_RATIO=";
+
     val.append(std::to_string(compac_opts.size_ratio));
     val.append("; MIN_MERGE_WIDTH=");
     val.append(std::to_string(compac_opts.min_merge_width));
@@ -552,6 +650,7 @@ static int rdb_i_s_cfoptions_fill_table(my_core::THD *const thd,
     val.append("; COMPRESSION_SIZE_PERCENT=");
     val.append(std::to_string(compac_opts.compression_size_percent));
     val.append("; STOP_STYLE=");
+
     switch (compac_opts.stop_style) {
     case rocksdb::kCompactionStopStyleSimilarSize:
       val.append("kCompactionStopStyleSimilarSize}");
@@ -562,6 +661,7 @@ static int rdb_i_s_cfoptions_fill_table(my_core::THD *const thd,
     default:
       val.append("}");
     }
+
     cf_option_types.push_back({"COMPACTION_OPTIONS_UNIVERSAL", val});
 
     // get COMPACTION_OPTION_FIFO option
@@ -569,96 +669,24 @@ static int rdb_i_s_cfoptions_fill_table(my_core::THD *const thd,
         {"COMPACTION_OPTION_FIFO::MAX_TABLE_FILES_SIZE",
          std::to_string(opts.compaction_options_fifo.max_table_files_size)});
 
-    // get block-based table related options
-    const rocksdb::BlockBasedTableOptions &table_options =
-        rdb_get_table_options();
+    // get table related options
+    std::vector<std::string> table_options =
+        split_into_vector(opts.table_factory->GetPrintableTableOptions(), '\n');
 
-    // get BLOCK_BASED_TABLE_FACTORY::CACHE_INDEX_AND_FILTER_BLOCKS option
-    cf_option_types.push_back(
-        {"BLOCK_BASED_TABLE_FACTORY::CACHE_INDEX_AND_FILTER_BLOCKS",
-         table_options.cache_index_and_filter_blocks ? "1" : "0"});
+    for (auto option : table_options) {
+      option.erase(std::remove(option.begin(), option.end(), ' '),
+                   option.end());
 
-    // get BLOCK_BASED_TABLE_FACTORY::INDEX_TYPE option value
-    switch (table_options.index_type) {
-    case rocksdb::BlockBasedTableOptions::kBinarySearch:
-      val = "kBinarySearch";
-      break;
-    case rocksdb::BlockBasedTableOptions::kHashSearch:
-      val = "kHashSearch";
-      break;
-    default:
-      val = "NULL";
+      int pos = option.find(":");
+      std::string option_name = option.substr(0, pos);
+      std::string option_value = option.substr(pos + 1, option.length());
+      std::transform(option_name.begin(), option_name.end(),
+                     option_name.begin(),
+                     [](unsigned char c) { return std::toupper(c); });
+
+      cf_option_types.push_back(
+          {"TABLE_FACTORY::" + option_name, option_value});
     }
-    cf_option_types.push_back({"BLOCK_BASED_TABLE_FACTORY::INDEX_TYPE", val});
-
-    // get BLOCK_BASED_TABLE_FACTORY::HASH_INDEX_ALLOW_COLLISION option value
-    cf_option_types.push_back(
-        {"BLOCK_BASED_TABLE_FACTORY::HASH_INDEX_ALLOW_COLLISION",
-         table_options.hash_index_allow_collision ? "ON" : "OFF"});
-
-    // get BLOCK_BASED_TABLE_FACTORY::CHECKSUM option value
-    switch (table_options.checksum) {
-    case rocksdb::kNoChecksum:
-      val = "kNoChecksum";
-      break;
-    case rocksdb::kCRC32c:
-      val = "kCRC32c";
-      break;
-    case rocksdb::kxxHash:
-      val = "kxxHash";
-      break;
-    default:
-      val = "NULL";
-    }
-    cf_option_types.push_back({"BLOCK_BASED_TABLE_FACTORY::CHECKSUM", val});
-
-    // get BLOCK_BASED_TABLE_FACTORY::NO_BLOCK_CACHE option value
-    cf_option_types.push_back({"BLOCK_BASED_TABLE_FACTORY::NO_BLOCK_CACHE",
-                               table_options.no_block_cache ? "ON" : "OFF"});
-
-    // get BLOCK_BASED_TABLE_FACTORY::FILTER_POLICY option
-    cf_option_types.push_back(
-        {"BLOCK_BASED_TABLE_FACTORY::FILTER_POLICY",
-         table_options.filter_policy == nullptr
-             ? "NULL"
-             : std::string(table_options.filter_policy->Name())});
-
-    // get BLOCK_BASED_TABLE_FACTORY::WHOLE_KEY_FILTERING option
-    cf_option_types.push_back({"BLOCK_BASED_TABLE_FACTORY::WHOLE_KEY_FILTERING",
-                               table_options.whole_key_filtering ? "1" : "0"});
-
-    // get BLOCK_BASED_TABLE_FACTORY::BLOCK_CACHE option
-    cf_option_types.push_back(
-        {"BLOCK_BASED_TABLE_FACTORY::BLOCK_CACHE",
-         table_options.block_cache == nullptr
-             ? "NULL"
-             : std::to_string(table_options.block_cache->GetUsage())});
-
-    // get BLOCK_BASED_TABLE_FACTORY::BLOCK_CACHE_COMPRESSED option
-    cf_option_types.push_back(
-        {"BLOCK_BASED_TABLE_FACTORY::BLOCK_CACHE_COMPRESSED",
-         table_options.block_cache_compressed == nullptr
-             ? "NULL"
-             : std::to_string(
-                   table_options.block_cache_compressed->GetUsage())});
-
-    // get BLOCK_BASED_TABLE_FACTORY::BLOCK_SIZE option
-    cf_option_types.push_back({"BLOCK_BASED_TABLE_FACTORY::BLOCK_SIZE",
-                               std::to_string(table_options.block_size)});
-
-    // get BLOCK_BASED_TABLE_FACTORY::BLOCK_SIZE_DEVIATION option
-    cf_option_types.push_back(
-        {"BLOCK_BASED_TABLE_FACTORY::BLOCK_SIZE_DEVIATION",
-         std::to_string(table_options.block_size_deviation)});
-
-    // get BLOCK_BASED_TABLE_FACTORY::BLOCK_RESTART_INTERVAL option
-    cf_option_types.push_back(
-        {"BLOCK_BASED_TABLE_FACTORY::BLOCK_RESTART_INTERVAL",
-         std::to_string(table_options.block_restart_interval)});
-
-    // get BLOCK_BASED_TABLE_FACTORY::FORMAT_VERSION option
-    cf_option_types.push_back({"BLOCK_BASED_TABLE_FACTORY::FORMAT_VERSION",
-                               std::to_string(table_options.format_version)});
 
     for (const auto &cf_option_type : cf_option_types) {
       DBUG_ASSERT(tables->table != nullptr);
@@ -673,12 +701,15 @@ static int rdb_i_s_cfoptions_fill_table(my_core::THD *const thd,
           cf_option_type.second.c_str(), cf_option_type.second.size(),
           system_charset_info);
 
-      ret = my_core::schema_table_store_record(thd, tables->table);
+      ret = static_cast<int>(
+          my_core::schema_table_store_record(thd, tables->table));
 
-      if (ret)
+      if (ret) {
         DBUG_RETURN(ret);
+      }
     }
   }
+
   DBUG_RETURN(0);
 }
 
@@ -738,6 +769,13 @@ static int rdb_i_s_global_info_fill_table(my_core::THD *const thd,
   static const uint32_t CF_ID_INDEX_BUF_LEN = 60;
 
   int ret = 0;
+  Rdb_hton_init_state::Scoped_lock state_lock(*rdb_get_hton_init_state(),
+                                              false);
+  if (!rdb_get_hton_init_state()->initialized()) {
+    ret = ER_PLUGIN_IS_NOT_LOADED;
+    my_error(ret, MYF(0), rocksdb_hton_name);
+    DBUG_RETURN(ret);
+  }
 
   /* max index info */
   const Rdb_dict_manager *const dict_manager = rdb_get_dict_manager();
@@ -748,6 +786,7 @@ static int rdb_i_s_global_info_fill_table(my_core::THD *const thd,
 
   if (dict_manager->get_max_index_id(&max_index_id)) {
     snprintf(max_index_id_buf, INT_BUF_LEN, "%u", max_index_id);
+
     ret |= rdb_global_info_fill_row(thd, tables, "MAX_INDEX_ID", "MAX_INDEX_ID",
                                     max_index_id_buf);
   }
@@ -788,14 +827,17 @@ static int rdb_i_s_global_info_fill_table(my_core::THD *const thd,
   dict_manager->get_ongoing_index_operation(
       &gl_index_ids, Rdb_key_def::DDL_DROP_INDEX_ONGOING);
   char cf_id_index_buf[CF_ID_INDEX_BUF_LEN] = {0};
+
   for (auto gl_index_id : gl_index_ids) {
     snprintf(cf_id_index_buf, CF_ID_INDEX_BUF_LEN, "cf_id:%u,index_id:%u",
              gl_index_id.cf_id, gl_index_id.index_id);
+
     ret |= rdb_global_info_fill_row(thd, tables, "DDL_DROP_INDEX_ONGOING",
                                     cf_id_index_buf, "");
 
-    if (ret)
+    if (ret) {
       break;
+    }
   }
 
   DBUG_RETURN(ret);
@@ -814,43 +856,58 @@ static int rdb_i_s_compact_stats_fill_table(my_core::THD *thd,
   DBUG_ENTER_FUNC();
 
   int ret = 0;
+  Rdb_hton_init_state::Scoped_lock state_lock(*rdb_get_hton_init_state(),
+                                              false);
+  if (!rdb_get_hton_init_state()->initialized()) {
+    ret = ER_PLUGIN_IS_NOT_LOADED;
+    my_error(ret, MYF(0), rocksdb_hton_name);
+    DBUG_RETURN(ret);
+  }
 
   rocksdb::DB *rdb = rdb_get_rocksdb_db();
+
+  if (!rdb) {
+    DBUG_RETURN(ret);
+  }
+
   Rdb_cf_manager &cf_manager = rdb_get_cf_manager();
-  DBUG_ASSERT(rdb != nullptr);
 
   for (auto cf_name : cf_manager.get_cf_names()) {
-    rocksdb::ColumnFamilyHandle *cfh;
-    bool is_automatic;
-    /*
-       Only the cf name is important. Whether it was generated automatically
-       does not matter, so is_automatic is ignored.
-    */
-    cfh = cf_manager.get_cf(cf_name.c_str(), "", nullptr, &is_automatic);
+    rocksdb::ColumnFamilyHandle *cfh = cf_manager.get_cf(cf_name);
+
     if (cfh == nullptr) {
       continue;
     }
-    std::map<std::string, double> props;
+
+    std::map<std::string, std::string> props;
     bool bool_ret MY_ATTRIBUTE((__unused__));
     bool_ret = rdb->GetMapProperty(cfh, "rocksdb.cfstats", &props);
     DBUG_ASSERT(bool_ret);
 
+    const std::string prop_name_prefix = "compaction.";
     for (auto const &prop_ent : props) {
       std::string prop_name = prop_ent.first;
-      double value = prop_ent.second;
-      std::size_t del_pos = prop_name.find('.');
+      if (prop_name.find(prop_name_prefix) != 0) {
+        continue;
+      }
+      std::string value = prop_ent.second;
+      std::size_t del_pos = prop_name.find('.', prop_name_prefix.size());
       DBUG_ASSERT(del_pos != std::string::npos);
-      std::string level_str = prop_name.substr(0, del_pos);
+      std::string level_str = prop_name.substr(
+          prop_name_prefix.size(), del_pos - prop_name_prefix.size());
       std::string type_str = prop_name.substr(del_pos + 1);
 
       Field **field = tables->table->field;
       DBUG_ASSERT(field != nullptr);
+
       field[0]->store(cf_name.c_str(), cf_name.size(), system_charset_info);
       field[1]->store(level_str.c_str(), level_str.size(), system_charset_info);
       field[2]->store(type_str.c_str(), type_str.size(), system_charset_info);
-      field[3]->store(value, true);
+      field[3]->store(std::stod(value));
+
+      ret |= static_cast<int>(
+          my_core::schema_table_store_record(thd, tables->table));
 
-      ret |= my_core::schema_table_store_record(thd, tables->table);
       if (ret != 0) {
         DBUG_RETURN(ret);
       }
@@ -890,7 +947,10 @@ enum {
   INDEX_NUMBER,
   INDEX_TYPE,
   KV_FORMAT_VERSION,
-  CF
+  TTL_DURATION,
+  INDEX_FLAGS,
+  CF,
+  AUTO_INCREMENT
 };
 } // namespace RDB_DDL_FIELD
 
@@ -905,7 +965,11 @@ static ST_FIELD_INFO rdb_i_s_ddl_fields_info[] = {
     ROCKSDB_FIELD_INFO("INDEX_TYPE", sizeof(uint16_t), MYSQL_TYPE_SHORT, 0),
     ROCKSDB_FIELD_INFO("KV_FORMAT_VERSION", sizeof(uint16_t), MYSQL_TYPE_SHORT,
                        0),
+    ROCKSDB_FIELD_INFO("TTL_DURATION", sizeof(uint64), MYSQL_TYPE_LONGLONG, 0),
+    ROCKSDB_FIELD_INFO("INDEX_FLAGS", sizeof(uint64), MYSQL_TYPE_LONGLONG, 0),
     ROCKSDB_FIELD_INFO("CF", NAME_LEN + 1, MYSQL_TYPE_STRING, 0),
+    ROCKSDB_FIELD_INFO("AUTO_INCREMENT", sizeof(uint64_t), MYSQL_TYPE_LONGLONG,
+                       MY_I_S_MAYBE_NULL | MY_I_S_UNSIGNED),
     ROCKSDB_FIELD_INFO_END};
 
 int Rdb_ddl_scanner::add_table(Rdb_tbl_def *tdef) {
@@ -916,6 +980,7 @@ int Rdb_ddl_scanner::add_table(Rdb_tbl_def *tdef) {
   DBUG_ASSERT(m_table != nullptr);
   Field **field = m_table->field;
   DBUG_ASSERT(field != nullptr);
+  const Rdb_dict_manager *dict_manager = rdb_get_dict_manager();
 
   const std::string &dbname = tdef->base_dbname();
   field[RDB_DDL_FIELD::TABLE_SCHEMA]->store(dbname.c_str(), dbname.size(),
@@ -946,10 +1011,20 @@ int Rdb_ddl_scanner::add_table(Rdb_tbl_def *tdef) {
     field[RDB_DDL_FIELD::INDEX_TYPE]->store(kd.m_index_type, true);
     field[RDB_DDL_FIELD::KV_FORMAT_VERSION]->store(kd.m_kv_format_version,
                                                    true);
+    field[RDB_DDL_FIELD::TTL_DURATION]->store(kd.m_ttl_duration, true);
+    field[RDB_DDL_FIELD::INDEX_FLAGS]->store(kd.m_index_flags_bitmap, true);
 
     std::string cf_name = kd.get_cf()->GetName();
     field[RDB_DDL_FIELD::CF]->store(cf_name.c_str(), cf_name.size(),
                                     system_charset_info);
+    ulonglong auto_incr;
+    if (dict_manager->get_auto_incr_val(tdef->get_autoincr_gl_index_id(),
+                                        &auto_incr)) {
+      field[RDB_DDL_FIELD::AUTO_INCREMENT]->set_notnull();
+      field[RDB_DDL_FIELD::AUTO_INCREMENT]->store(auto_incr, true);
+    } else {
+      field[RDB_DDL_FIELD::AUTO_INCREMENT]->set_null();
+    }
 
     ret = my_core::schema_table_store_record(m_thd, m_table);
     if (ret)
@@ -965,14 +1040,32 @@ static int rdb_i_s_ddl_fill_table(my_core::THD *const thd,
 
   DBUG_ASSERT(thd != nullptr);
   DBUG_ASSERT(tables != nullptr);
+  DBUG_ASSERT(tables->table != nullptr);
+
+  int ret = 0;
+  Rdb_hton_init_state::Scoped_lock state_lock(*rdb_get_hton_init_state(),
+                                              false);
+  if (!rdb_get_hton_init_state()->initialized()) {
+    ret = ER_PLUGIN_IS_NOT_LOADED;
+    my_error(ret, MYF(0), rocksdb_hton_name);
+    DBUG_RETURN(ret);
+  }
+
+  rocksdb::DB *const rdb = rdb_get_rocksdb_db();
+
+  if (!rdb) {
+    DBUG_RETURN(ret);
+  }
 
   Rdb_ddl_scanner ddl_arg;
+
   ddl_arg.m_thd = thd;
   ddl_arg.m_table = tables->table;
 
   Rdb_ddl_manager *ddl_manager = rdb_get_ddl_manager();
   DBUG_ASSERT(ddl_manager != nullptr);
-  int ret = ddl_manager->scan_for_tables(&ddl_arg);
+
+  ret = ddl_manager->scan_for_tables(&ddl_arg);
 
   DBUG_RETURN(ret);
 }
@@ -1103,19 +1196,32 @@ static int rdb_i_s_index_file_map_fill_table(my_core::THD *const thd,
   DBUG_ASSERT(tables->table != nullptr);
 
   int ret = 0;
+  Rdb_hton_init_state::Scoped_lock state_lock(*rdb_get_hton_init_state(),
+                                              false);
+  if (!rdb_get_hton_init_state()->initialized()) {
+    ret = ER_PLUGIN_IS_NOT_LOADED;
+    my_error(ret, MYF(0), rocksdb_hton_name);
+    DBUG_RETURN(ret);
+  }
+
   Field **field = tables->table->field;
   DBUG_ASSERT(field != nullptr);
 
   /* Iterate over all the column families */
   rocksdb::DB *const rdb = rdb_get_rocksdb_db();
-  DBUG_ASSERT(rdb != nullptr);
+
+  if (!rdb) {
+    DBUG_RETURN(ret);
+  }
 
   const Rdb_cf_manager &cf_manager = rdb_get_cf_manager();
+
   for (const auto &cf_handle : cf_manager.get_all_cf()) {
     /* Grab the the properties of all the tables in the column family */
     rocksdb::TablePropertiesCollection table_props_collection;
     const rocksdb::Status s =
         rdb->GetPropertiesOfAllTables(cf_handle, &table_props_collection);
+
     if (!s.ok()) {
       continue;
     }
@@ -1125,12 +1231,14 @@ static int rdb_i_s_index_file_map_fill_table(my_core::THD *const thd,
     for (const auto &props : table_props_collection) {
       /* Add the SST name into the output */
       const std::string sst_name = rdb_filename_without_path(props.first);
+
       field[RDB_INDEX_FILE_MAP_FIELD::SST_NAME]->store(
           sst_name.data(), sst_name.size(), system_charset_info);
 
       /* Get the __indexstats__ data out of the table property */
       std::vector<Rdb_index_stats> stats;
       Rdb_tbl_prop_coll::read_stats_from_tbl_props(props.second, &stats);
+
       if (stats.empty()) {
         field[RDB_INDEX_FILE_MAP_FIELD::COLUMN_FAMILY]->store(-1, true);
         field[RDB_INDEX_FILE_MAP_FIELD::INDEX_NUMBER]->store(-1, true);
@@ -1141,7 +1249,7 @@ static int rdb_i_s_index_file_map_fill_table(my_core::THD *const thd,
         field[RDB_INDEX_FILE_MAP_FIELD::ENTRY_MERGES]->store(-1, true);
         field[RDB_INDEX_FILE_MAP_FIELD::ENTRY_OTHERS]->store(-1, true);
       } else {
-        for (auto it : stats) {
+        for (const auto &it : stats) {
           /* Add the index number, the number of rows, and data size to the
            * output */
           field[RDB_INDEX_FILE_MAP_FIELD::COLUMN_FAMILY]->store(
@@ -1159,12 +1267,14 @@ static int rdb_i_s_index_file_map_fill_table(my_core::THD *const thd,
               it.m_entry_merges, true);
           field[RDB_INDEX_FILE_MAP_FIELD::ENTRY_OTHERS]->store(
               it.m_entry_others, true);
+
           std::string distinct_keys_prefix;
 
           for (size_t i = 0; i < it.m_distinct_keys_per_prefix.size(); i++) {
             if (i > 0) {
               distinct_keys_prefix += ",";
             }
+
             distinct_keys_prefix +=
                 std::to_string(it.m_distinct_keys_per_prefix[i]);
           }
@@ -1174,7 +1284,9 @@ static int rdb_i_s_index_file_map_fill_table(my_core::THD *const thd,
               system_charset_info);
 
           /* Tell MySQL about this row in the virtual table */
-          ret = my_core::schema_table_store_record(thd, tables->table);
+          ret = static_cast<int>(
+              my_core::schema_table_store_record(thd, tables->table));
+
           if (ret != 0) {
             break;
           }
@@ -1227,11 +1339,22 @@ static int rdb_i_s_lock_info_fill_table(my_core::THD *const thd,
   DBUG_ASSERT(thd != nullptr);
   DBUG_ASSERT(tables != nullptr);
   DBUG_ASSERT(tables->table != nullptr);
+  DBUG_ASSERT(tables->table->field != nullptr);
 
   int ret = 0;
+  Rdb_hton_init_state::Scoped_lock state_lock(*rdb_get_hton_init_state(),
+                                              false);
+  if (!rdb_get_hton_init_state()->initialized()) {
+    ret = ER_PLUGIN_IS_NOT_LOADED;
+    my_error(ret, MYF(0), rocksdb_hton_name);
+    DBUG_RETURN(ret);
+  }
 
   rocksdb::TransactionDB *const rdb = rdb_get_rocksdb_db();
-  DBUG_ASSERT(rdb != nullptr);
+
+  if (!rdb) {
+    DBUG_RETURN(ret);
+  }
 
   /* cf id -> rocksdb::KeyLockInfo */
   std::unordered_multimap<uint32_t, rocksdb::KeyLockInfo> lock_info =
@@ -1254,12 +1377,15 @@ static int rdb_i_s_lock_info_fill_table(my_core::THD *const thd,
           key_lock_info.exclusive ? "X" : "S", 1, system_charset_info);
 
       /* Tell MySQL about this row in the virtual table */
-      ret = my_core::schema_table_store_record(thd, tables->table);
+      ret = static_cast<int>(
+          my_core::schema_table_store_record(thd, tables->table));
+
       if (ret != 0) {
         break;
       }
     }
   }
+
   DBUG_RETURN(ret);
 }
 
@@ -1335,8 +1461,22 @@ static int rdb_i_s_trx_info_fill_table(my_core::THD *const thd,
   DBUG_ASSERT(thd != nullptr);
   DBUG_ASSERT(tables != nullptr);
   DBUG_ASSERT(tables->table != nullptr);
+  DBUG_ASSERT(tables->table->field != nullptr);
 
   int ret = 0;
+  Rdb_hton_init_state::Scoped_lock state_lock(*rdb_get_hton_init_state(),
+                                              false);
+  if (!rdb_get_hton_init_state()->initialized()) {
+    ret = ER_PLUGIN_IS_NOT_LOADED;
+    my_error(ret, MYF(0), rocksdb_hton_name);
+    DBUG_RETURN(ret);
+  }
+
+  rocksdb::DB *const rdb = rdb_get_rocksdb_db();
+
+  if (!rdb) {
+    DBUG_RETURN(ret);
+  }
 
   const std::vector<Rdb_trx_info> &all_trx_info = rdb_get_all_trx_info();
 
@@ -1345,6 +1485,7 @@ static int rdb_i_s_trx_info_fill_table(my_core::THD *const thd,
         rdb_hexdump(info.name.c_str(), info.name.length(), NAME_LEN);
     auto key_hexstr = rdb_hexdump(info.waiting_key.c_str(),
                                   info.waiting_key.length(), FN_REFLEN);
+
     tables->table->field[RDB_TRX_FIELD::TRANSACTION_ID]->store(info.trx_id,
                                                                true);
     tables->table->field[RDB_TRX_FIELD::STATE]->store(
@@ -1376,7 +1517,9 @@ static int rdb_i_s_trx_info_fill_table(my_core::THD *const thd,
         info.query_str.c_str(), info.query_str.length(), system_charset_info);
 
     /* Tell MySQL about this row in the virtual table */
-    ret = my_core::schema_table_store_record(thd, tables->table);
+    ret = static_cast<int>(
+        my_core::schema_table_store_record(thd, tables->table));
+
     if (ret != 0) {
       break;
     }
@@ -1401,6 +1544,125 @@ static int rdb_i_s_trx_info_init(void *const p) {
   DBUG_RETURN(0);
 }
 
+/*
+  Support for INFORMATION_SCHEMA.ROCKSDB_DEADLOCK dynamic table
+ */
+namespace RDB_DEADLOCK_FIELD {
+enum {
+  DEADLOCK_ID = 0,
+  TRANSACTION_ID,
+  CF_NAME,
+  WAITING_KEY,
+  LOCK_TYPE,
+  INDEX_NAME,
+  TABLE_NAME,
+  ROLLED_BACK
+};
+} // namespace RDB_TRX_FIELD
+
+static ST_FIELD_INFO rdb_i_s_deadlock_info_fields_info[] = {
+    ROCKSDB_FIELD_INFO("DEADLOCK_ID", sizeof(ulonglong), MYSQL_TYPE_LONGLONG,
+                       0),
+    ROCKSDB_FIELD_INFO("TRANSACTION_ID", sizeof(ulonglong), MYSQL_TYPE_LONGLONG,
+                       0),
+    ROCKSDB_FIELD_INFO("CF_NAME", NAME_LEN + 1, MYSQL_TYPE_STRING, 0),
+    ROCKSDB_FIELD_INFO("WAITING_KEY", FN_REFLEN + 1, MYSQL_TYPE_STRING, 0),
+    ROCKSDB_FIELD_INFO("LOCK_TYPE", NAME_LEN + 1, MYSQL_TYPE_STRING, 0),
+    ROCKSDB_FIELD_INFO("INDEX_NAME", NAME_LEN + 1, MYSQL_TYPE_STRING, 0),
+    ROCKSDB_FIELD_INFO("TABLE_NAME", NAME_LEN + 1, MYSQL_TYPE_STRING, 0),
+    ROCKSDB_FIELD_INFO("ROLLED_BACK", sizeof(ulonglong), MYSQL_TYPE_LONGLONG,
+                       0),
+    ROCKSDB_FIELD_INFO_END};
+
+/* Fill the information_schema.rocksdb_trx virtual table */
+static int rdb_i_s_deadlock_info_fill_table(
+    my_core::THD *const thd, my_core::TABLE_LIST *const tables,
+    my_core::Item *const cond MY_ATTRIBUTE((__unused__))) {
+  DBUG_ENTER_FUNC();
+
+  DBUG_ASSERT(thd != nullptr);
+  DBUG_ASSERT(tables != nullptr);
+  DBUG_ASSERT(tables->table != nullptr);
+  DBUG_ASSERT(tables->table->field != nullptr);
+
+  static const std::string str_exclusive("EXCLUSIVE");
+  static const std::string str_shared("SHARED");
+
+  int ret = 0;
+  Rdb_hton_init_state::Scoped_lock state_lock(*rdb_get_hton_init_state(),
+                                              false);
+  if (!rdb_get_hton_init_state()->initialized()) {
+    ret = ER_PLUGIN_IS_NOT_LOADED;
+    my_error(ret, MYF(0), rocksdb_hton_name);
+    DBUG_RETURN(ret);
+  }
+
+  rocksdb::DB *const rdb = rdb_get_rocksdb_db();
+
+  if (!rdb) {
+    DBUG_RETURN(ret);
+  }
+
+  const std::vector<Rdb_deadlock_info> &all_dl_info = rdb_get_deadlock_info();
+
+  ulonglong id = 0;
+  for (const auto &info : all_dl_info) {
+    for (const auto &trx_info : info.path) {
+      tables->table->field[RDB_DEADLOCK_FIELD::DEADLOCK_ID]->store(id, true);
+      tables->table->field[RDB_DEADLOCK_FIELD::TRANSACTION_ID]->store(
+          trx_info.trx_id, true);
+      tables->table->field[RDB_DEADLOCK_FIELD::CF_NAME]->store(
+          trx_info.cf_name.c_str(), trx_info.cf_name.length(),
+          system_charset_info);
+      tables->table->field[RDB_DEADLOCK_FIELD::WAITING_KEY]->store(
+          trx_info.waiting_key.c_str(), trx_info.waiting_key.length(),
+          system_charset_info);
+      if (trx_info.exclusive_lock) {
+        tables->table->field[RDB_DEADLOCK_FIELD::LOCK_TYPE]->store(
+            str_exclusive.c_str(), str_exclusive.length(), system_charset_info);
+      } else {
+        tables->table->field[RDB_DEADLOCK_FIELD::LOCK_TYPE]->store(
+            str_shared.c_str(), str_shared.length(), system_charset_info);
+      }
+      tables->table->field[RDB_DEADLOCK_FIELD::INDEX_NAME]->store(
+          trx_info.index_name.c_str(), trx_info.index_name.length(),
+          system_charset_info);
+      tables->table->field[RDB_DEADLOCK_FIELD::TABLE_NAME]->store(
+          trx_info.table_name.c_str(), trx_info.table_name.length(),
+          system_charset_info);
+      tables->table->field[RDB_DEADLOCK_FIELD::ROLLED_BACK]->store(
+          trx_info.trx_id == info.victim_trx_id, true);
+
+      /* Tell MySQL about this row in the virtual table */
+      ret = static_cast<int>(
+          my_core::schema_table_store_record(thd, tables->table));
+
+      if (ret != 0) {
+        break;
+      }
+    }
+    id++;
+  }
+
+  DBUG_RETURN(ret);
+}
+
+/* Initialize the information_schema.rocksdb_trx_info virtual table */
+static int rdb_i_s_deadlock_info_init(void *const p) {
+  DBUG_ENTER_FUNC();
+
+  DBUG_ASSERT(p != nullptr);
+
+  my_core::ST_SCHEMA_TABLE *schema;
+
+  schema = (my_core::ST_SCHEMA_TABLE *)p;
+
+  schema->fields_info = rdb_i_s_deadlock_info_fields_info;
+  schema->fill_table = rdb_i_s_deadlock_info_fill_table;
+
+  DBUG_RETURN(0);
+}
+
 static int rdb_i_s_deinit(void *p MY_ATTRIBUTE((__unused__))) {
   DBUG_ENTER_FUNC();
   DBUG_RETURN(0);
@@ -1584,4 +1846,20 @@ struct st_mysql_plugin rdb_i_s_trx_info = {
     nullptr, /* config options */
     0,       /* flags */
 };
+
+struct st_mysql_plugin rdb_i_s_deadlock_info = {
+    MYSQL_INFORMATION_SCHEMA_PLUGIN,
+    &rdb_i_s_info,
+    "ROCKSDB_DEADLOCK",
+    "Facebook",
+    "RocksDB transaction information",
+    PLUGIN_LICENSE_GPL,
+    rdb_i_s_deadlock_info_init,
+    nullptr,
+    0x0001,  /* version number (0.1) */
+    nullptr, /* status variables */
+    nullptr, /* system variables */
+    nullptr, /* config options */
+    0,       /* flags */
+};
 } // namespace myrocks
